package jp.co.asterisk.asreader.l251g.support.adapter;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;

import jp.co.asterisk.asreader.l251g.R;
import com.asreader.lib.barcode.BarcodeType;

import android.content.Context;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

public class BarcodeListAdapter extends BaseAdapter {

	private LayoutInflater mInflater;
	private ArrayList<BarcodeListItem> mList;
	private HashMap<String, BarcodeListItem> mMap;

	public BarcodeListAdapter(Context context) {
		super();
		
		this.mInflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		this.mList = new ArrayList<>();
		this.mMap = new HashMap<>();
	}

	public void clear() {
		this.mList.clear();
		this.notifyDataSetChanged();
		this.mMap.clear();
	}

	// DB
	public void addItem(BarcodeType type,int count, String codeId, String barcode, long time) {
		BarcodeListItem item;
		String tag = type + codeId + barcode;
		if (null == (item = mMap.get(tag))) {
			item = new BarcodeListItem(type, count, codeId, barcode, time);
			mList.add(item);
			mMap.put(tag, item);
		} else {
			item.updateItem(count, time);
		}
		
		notifyDataSetChanged();
	}

	public void addItem(BarcodeType type, String codeId, String barcode, long time) {
		BarcodeListItem item;
		String tag = type + codeId + barcode;
		if (null == (item = mMap.get(tag))) {
			item = new BarcodeListItem(type, codeId, barcode, time);
			mList.add(item);
			mMap.put(tag, item);
		} else {
			item.updateItem(time);
		}

		notifyDataSetChanged();
	}
	
	@Override
	public int getCount() {
		return this.mList.size();
	}

	@Override
	public String getItem(int position) {
		return this.mList.get(position).getBarcode();
	}

	@Override
	public long getItemId(int position) {
		return position;
	}
	
	public String getItemForCsv(int position) {
		BarcodeListItem item = mList.get(position);
		return String.format("%s,%s", item.getBarcode(), item.getTime());
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		BarcodeListViewHolder holder;
		
		if (null == convertView) {
			convertView = this.mInflater.inflate(R.layout.item_barcode_list, parent, false);
			holder = new BarcodeListViewHolder(convertView);
		} else {
			holder = (BarcodeListViewHolder)convertView.getTag();
		}
		holder.setItem(this.mList.get(position));
		return convertView;
	}

	// DB
	public String[] getBarcodeItemForDB(int position)
	{
		BarcodeListItem item = mList.get(position);

		String[] temp = new String[5];
		temp[0] = item.getBarcode();
		temp[1] = String.valueOf(item.getCount());
		temp[2] = String.valueOf(item.getType());
		temp[3] = item.getCodeId();
		temp[4] = String.valueOf(item.mTime);

		//Barcode, count, type, codeid, time
		return temp;
	}

	private class BarcodeListItem {
		private BarcodeType mType;
		private String mCodeId;
		private String mBarcode;
		private int mCount;
		private long mTime;
		
		private SimpleDateFormat mDateFormat;

		// DB
		private BarcodeListItem(BarcodeType type,int count, String codeId, String barcode, long time) {
			mType = type;
			mCodeId = codeId;
			mBarcode = barcode;
			mCount = count;
			mTime = time;

			mDateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss", Locale.US);
		}

		private BarcodeListItem(BarcodeType type, String codeId, String barcode, long time) {
			mType = type;
			mCodeId = codeId;
			mBarcode = barcode;
			mCount = 1;
			mTime = time;

			mDateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss", Locale.US);
		}

		public BarcodeType getType() {
			return mType;
		}
		
		private String getCodeId() {
			return mCodeId;
		}
		
		public String getBarcode() {
			return mBarcode;
		}
		
		public int getCount() {
			return mCount;
		}
		
		private void updateItem(long time) {
			mCount++;
			mTime = time;
		}

		// DB
		private void updateItem(int count, long time) {
			mCount = count;
			mTime = time;
		}
		
		public String getTime() {
			return mDateFormat.format(new Date(mTime));
		}
	}

	private class BarcodeListViewHolder {
		private TextView txtType;
		private TextView txtCodeId;
		private TextView txtBarcode;
		private TextView txtCount;

		private BarcodeListViewHolder(View parent) {
			txtType = (TextView) parent.findViewById(R.id.code_type);
			txtCodeId = (TextView)parent.findViewById(R.id.code_id);
			txtBarcode = (TextView) parent.findViewById(R.id.barcode);
			txtCount = (TextView) parent.findViewById(R.id.barcode_count);
			parent.setTag(this);
		}

		public void setItem(BarcodeListItem item) {
			txtType.setText(item.getType().toString());
			txtCodeId.setText(item.getCodeId());
			txtBarcode.setText(item.getBarcode());
			txtCount.setText(String.format(Locale.US, "%d", item.getCount()));
		}
	}
}
