package jp.co.asterisk.asreader.l251g.support.adapter;

import java.util.ArrayList;

import com.asreader.lib.barcode.spc.ParamValue;
import com.asreader.lib.barcode.spc.type.ParamName;
import jp.co.asterisk.asreader.l251g.R;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.TextView;

public class BarcodeSymbolListAdapter extends BaseAdapter {

	private LayoutInflater mInflater;
	private ArrayList<BarcodeSymbolListItem> mList;
	private final Object objLock = new Object();
	
	public BarcodeSymbolListAdapter(Context context) {
		super();
		mInflater = (LayoutInflater) context
				.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		mList = new ArrayList<>();
	}
	
	public void clear() {
		synchronized (objLock) {
			mList.clear();
			this.notifyDataSetChanged();
		}
	}
	
	public void addItem(ParamName name, boolean enabled) {
		synchronized (objLock) {
			mList.add(new BarcodeSymbolListItem(name, enabled));
		}
	}

	public void setItem(ParamName name, boolean enabled) {
		synchronized (objLock) {
			for(BarcodeSymbolListItem item : mList) {
				if(item.getName() == name)
					item.setEnabled(enabled);
			}
		}
	}
	
	public ParamValue getParamValue(int position) {
		ParamValue value;
		synchronized (objLock) {
			BarcodeSymbolListItem item = mList.get(position);
			if (item.getName() == ParamName.GS1Emulation)
				value = new ParamValue(item.getName(), item.getValue());
			else
				value = new ParamValue(item.getName(), item.isEnabled());
		}
		return value;
	}

	public void refreshAdapter() {
		this.notifyDataSetChanged();
	}

	public void enableAll() {
		synchronized (objLock) {
			for (BarcodeSymbolListItem item : mList) {
				item.setEnabled(true);
			}
		}
		notifyDataSetChanged();
	}
	
	public void disableAll() {
		synchronized (objLock) {
			for (BarcodeSymbolListItem item : mList) {
				item.setEnabled(false);
			}
		}
		notifyDataSetChanged();
	}
	
	public void defaultAll() {
		synchronized (objLock) {
			for (BarcodeSymbolListItem item : mList) {
				switch (item.getName()) {
					case AztecCode:
						item.setEnabled(true);
						break;
					case ChinaPost:
						item.setEnabled(false);
						break;
					case HanXinCode:
						item.setEnabled(false);
						break;
					case Codabar:
						item.setEnabled(true);
						break;
					case CodablockA:
						item.setEnabled(false);
						break;
					case CodablockF:
						item.setEnabled(false);
						break;
					case Code11:
						item.setEnabled(false);
						break;
					case Code128:
						item.setEnabled(true);
						break;
					case Code32Pharmaceutical:
						item.setEnabled(false);
						break;
					case Code39:
						item.setEnabled(true);
						break;
					case Code93:
						item.setEnabled(true);
						break;
					case Matrix:
						item.setEnabled(true);
						break;
					case EAN13:
						item.setEnabled(true);
						break;
					case EAN8:
						item.setEnabled(true);
						break;
					case GS1CompositeCodes:
						item.setEnabled(false);
						break;
					case GS1DataBarExpanded:
						item.setEnabled(true);
						break;
					case GS1DataBarLimited:
						item.setEnabled(true);
						break;
					case GS1DataBarOmnidirectional:
						item.setEnabled(true);
						break;
					case GS1Emulation:
						item.setEnabled(false);
						break;
					case GS1128:
						item.setEnabled(true);
						break;
					case I2of5:
						item.setEnabled(true);
						break;
					case KoreaPost:
						item.setEnabled(false);
						break;
					case X2of5:
						item.setEnabled(false);
						break;
					case MaxiCode:
						item.setEnabled(false);
						break;
					case MicroPDF417:
						item.setEnabled(false);
						break;
					case MSI:
						item.setEnabled(false);
						break;
					case NEC2of5:
						item.setEnabled(true);
						break;
					case PostalCodes:
						item.setEnabled(false);
						break;
					case PDF417:
						item.setEnabled(true);
						break;
					case MacroPDF417:
						item.setEnabled(true);
						break;
					case QRCode:
						item.setEnabled(true);
						break;
					case A2of5:
						item.setEnabled(false);
						break;
					case R2of5:
						item.setEnabled(false);
						break;
					case TLC39:
						item.setEnabled(false);
						break;
					case Telepen:
						item.setEnabled(false);
						break;
					case TriopticCode:
						item.setEnabled(false);
						break;
					case UPCA:
						item.setEnabled(true);
						break;
					case UPCACouponCode:
						item.setEnabled(false);
						break;
					case UPCE0:
						item.setEnabled(true);
						break;
					case UPCE1:
						item.setEnabled(false);
						break;
				}
			}
		}
		notifyDataSetChanged();
	}
	
	@Override
	public int getCount() {
		int res;
		synchronized (objLock) {
			res = mList.size();
		}
		return res;
	}

	@Override
	public ParamName getItem(int position) {
		ParamName paramName;
		synchronized (objLock) {
			paramName = mList.get(position).getName();
		}
		return paramName;
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public View getView(final int position, View convertView, ViewGroup parent) {
		BarcodeSymbolListViewHolder holder;
		
		if (null == convertView) {
			convertView = mInflater.inflate(R.layout.item_barcode_symbol_list, parent, false);
			holder = new BarcodeSymbolListViewHolder(convertView);
		} else {
			holder = (BarcodeSymbolListViewHolder)convertView.getTag();
		}
		holder.setItem(position, mList.get(position));
		return convertView;
	}

	private class BarcodeSymbolListViewHolder implements OnCheckedChangeListener {

		private int mPosition;
		private CheckBox chkState;
		private TextView txtName;

		private BarcodeSymbolListViewHolder(View parent) {
			mPosition = -1;
			chkState = (CheckBox) parent.findViewById(R.id.cb_checkbox);
			chkState.setOnCheckedChangeListener(this);
			txtName = (TextView) parent.findViewById(R.id.name);
			parent.setTag(this);
		}

		public void setItem(int position, BarcodeSymbolListItem item) {
			mPosition = position;
			chkState.setChecked(item.isEnabled());
			txtName.setText(item.getName().toString());
		}

		@Override
		public void onCheckedChanged(CompoundButton buttonView,
				boolean isChecked) {
			BarcodeSymbolListItem item = mList.get(mPosition);
			if (item != null) {
				item.setEnabled(isChecked);
			}
		}
	}

	private class BarcodeSymbolListItem {

		private ParamName mName;
		private boolean mIsEnabled;
		private int mValue;

		private BarcodeSymbolListItem(ParamName name, boolean enabled) {
			mName = name;
			mIsEnabled = enabled;
			if(name == ParamName.GS1Emulation)
				mValue = enabled ? 3 : 0;
			else
				mValue = enabled ? 1 : 0;
		}

		public ParamName getName() {
			return mName;
		}

		public boolean isEnabled() {
			return mIsEnabled;
		}

		public int getValue() { return mValue; }
		
		public void setEnabled(boolean enabled) {
			mIsEnabled = enabled;
			if(mName == ParamName.GS1Emulation)
				mValue = enabled ? 3 : 0;
			else
				mValue = enabled ? 1 : 0;
		}
	}
}
