package jp.co.asterisk.asreader.l251g.support.data;

import android.app.AlertDialog;
import android.content.Context;
import android.media.MediaScannerConnection;
import android.net.Uri;
import android.os.Environment;
import android.util.Log;
import android.widget.BaseAdapter;

import jp.co.asterisk.asreader.l251g.R;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.ref.WeakReference;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import jp.co.asterisk.asreader.l251g.support.adapter.BarcodeListAdapter;
import jp.co.asterisk.asreader.l251g.support.adapter.TagListAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;

public class SaveAsExcel {
    private static final String TAG = SaveAsExcel.class.getSimpleName();
    private WeakReference<Context> mWeakContext;
    private Context context;
    private TagListAdapter mTagList = null;
    private BarcodeListAdapter mBarcodeList = null;
    private boolean mDisplayPc = false;
    private int mDataType = DATA_TYPE_UNKNOWN;
    private int mMaxCount = 0;

    public static final int DATA_TYPE_UNKNOWN = 0;
    public static final int DATA_TYPE_RFID = 1;
    public static final int DATA_TYPE_BARCODE = 2;

    private Workbook workbook;
    private Sheet sheet;
    private String mFileName = "";
    private String mColumnName = "";
    private File mFolder;
    private File mFile;


    public SaveAsExcel(Context context, int dataType, BaseAdapter data, boolean displayPc) {
        mWeakContext = new WeakReference<>(context);
        this.context = context;
        mDataType = dataType;
        mDisplayPc = displayPc;

        long now = System.currentTimeMillis();
        SimpleDateFormat simpleDate = new SimpleDateFormat("yyMMddHHmmss");
        String getTime = simpleDate.format(new Date(now));

        switch (mDataType) {
            case DATA_TYPE_RFID:
                mTagList = (TagListAdapter) data;
                mMaxCount = mTagList.getCount();
                mFileName = "/rfid_"+getTime+".xls";

                if (mTagList.getReportRSSI())
                    mColumnName = String.format(Locale.US, "%s,time,count,rssi", mDisplayPc ? "pcepc hex" : "epc hex");
                else
                    mColumnName = String.format(Locale.US, "%s,time,count", mDisplayPc ? "pcepc hex" : "epc hex");
                break;
            case DATA_TYPE_BARCODE:
                mBarcodeList = (BarcodeListAdapter) data;
                mMaxCount = mBarcodeList.getCount();
                mFileName = "/barcode_"+getTime+".xls";
                mColumnName = "barcode,time";
                break;
            default:
                mTagList = null;
                mBarcodeList = null;
                break;
        }
    }

    public void write() {
        mFolder = new File(Environment.getExternalStorageDirectory() + "/EXCEL_LOG");
        mFile = new File(mFolder, mFileName);

        if (mWeakContext == null)
            return;

        if (mDataType == DATA_TYPE_RFID && mTagList == null)
            return;

        if (mDataType == DATA_TYPE_BARCODE && mBarcodeList == null)
            return;

        WaitDialog.showProgess(mWeakContext.get(), R.string.saving_excel);
        WaitDialog.setMax(mMaxCount);

        boolean result = false;
        if (createExcel()) {

            FileOutputStream fos = null;

            if (!mFolder.exists()) {
                try {
                    mFolder.mkdir(); // 폴더가 없는 경우 생성
                } catch (Exception e) {
                    e.getStackTrace();
                }
            }

            try {
                fos = new FileOutputStream(mFile);
                workbook.write(fos);
                result = true;
            } catch (FileNotFoundException e) {
                e.printStackTrace();
                result = false;
            } catch (IOException e) {
                e.printStackTrace();
                result = false;
            } finally {
                try {
                    if (workbook != null) workbook.close();
                    if (fos != null) fos.close();

                    result = true;
                } catch (IOException e) {
                    e.printStackTrace();
                    result = false;
                }
            }
        }
        exportExcel(result);
    }

    private boolean createExcel() {
        workbook = new HSSFWorkbook();
        sheet = workbook.createSheet();

        Row row = sheet.createRow(0);
        Cell cell;

        String[] name = mColumnName.split(",");
        for (int i = 0; i < name.length; i++) {
            cell = row.createCell(i);
            cell.setCellValue(name[i]);
        }

        String data;
        String[] datalist;
        for (int i = 0; i < mMaxCount; i++) {
            switch (mDataType) {
                case DATA_TYPE_RFID:
                    data = mTagList.getItemForCsv(i, mDisplayPc);
                    break;
                case DATA_TYPE_BARCODE:
                    data = mBarcodeList.getItemForCsv(i);
                    if (data.contains("\r\n")) {
                        data = data.replace("\r\n", "");
                    }
                    break;
                default:
                    return false;
            }
            datalist = data.split(",");
            row = sheet.createRow(i + 1);

            for (int j = 0; j < datalist.length; j++) {
                cell = row.createCell(j);
                cell.setCellValue(datalist[j]);

                WaitDialog.setProgress(i);
            }
        }

        setUpdateFileList();

        return true;
    }

    private void exportExcel(boolean result) {
        WaitDialog.hide();

        AlertDialog.Builder builder = new AlertDialog.Builder(mWeakContext.get());
        builder.setMessage(result ? R.string.save_complete : R.string.failed_to_save);
        builder.setPositiveButton(R.string.action_ok, null);
        builder.setCancelable(true);
        builder.show();
    }


    /**
     * [Note]  Update saved log file list in directory (Android OS function)
     * @Method setUpdateFileList
     * @param  con : Context
     * @return none
     */
    private File mScanDir;
    private MediaScannerConnection mScanner;
    public void setUpdateFileList()
    {
        mScanDir = new File(mFolder.getAbsolutePath());
        mScanner = new MediaScannerConnection(context, new MediaScannerConnection.MediaScannerConnectionClient() {
            @Override
            public void onMediaScannerConnected() {
                File[] files = mScanDir.listFiles();
                if(files!=null){
                    for (File f : files) {
                        String path = f.getAbsolutePath();
                        mScanner.scanFile(path, null);
                        Log.i("onMediaScannerConnected", "Media Scan completed on file: path=" + path);
                    }
                }
            }

            @Override
            public void onScanCompleted(String path, Uri uri) {
                Log.i("onScanCompleted", "Media Scan completed on directory: path=" + path + " uri=" + uri);
                mScanner.disconnect();
                mScanner = null;
            }
        });
        mScanner.connect();
    }

}
