package jp.co.asterisk.asreader.l251g.support.util;

import android.content.Context;
import android.content.SharedPreferences;

import jp.co.asterisk.asreader.l251g.MainActivity;
import com.asreader.lib.AsReader;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.exception.AsReaderException;
import com.asreader.lib.rfid.params.SelectMaskEpcParam;
import com.asreader.lib.rfid.params.SelectMaskParam;
import com.asreader.lib.rfid.type.LbtItem;
import com.asreader.lib.rfid.type.MaskType;
import com.asreader.lib.rfid.type.QuerySL;
import com.asreader.lib.rfid.type.QuerySession;
import com.asreader.lib.rfid.type.QueryTarget;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import static com.asreader.lib.rfid.type.MaskType.EPC;
import static com.asreader.lib.rfid.type.MaskType.NoMask;
import static com.asreader.lib.rfid.type.MaskType.Selection;

/**
 * Created by Kastro on 2018-07-25.
 */

public class UserConfigs {

    private static final String TAG = UserConfigs.class.getSimpleName();
    private static ConfigParams _configParams = null;
    private final String KEY = "mask_params";

    private Context _context = null;
    private Gson _gson = null;

    public UserConfigs(Context context) {
        _context = context;
        _gson = new GsonBuilder().create();
    }

    public void Save() {
        String maskParams = _gson.toJson(_configParams, ConfigParams.class);
        SharedPreferences sharedPreferences = _context.getSharedPreferences(MainActivity.APP_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putString(KEY, maskParams);

        editor.apply();
    }
    public boolean Load(AsReader reader) {
        SharedPreferences sharedPreferences = _context.getSharedPreferences(MainActivity.APP_NAME, Context.MODE_PRIVATE);
        String maskParams = sharedPreferences.getString(KEY, "");
        if(maskParams.isEmpty()) {
            _configParams = new ConfigParams();
        } else {
            _configParams = _gson.fromJson(maskParams, ConfigParams.class);
        }

        return SaveToDevice(reader, _configParams);
    }
    private boolean SaveToDevice(AsReader reader, ConfigParams configParams) {

        try {
            if(configParams._maskType == null)
                configParams._maskType = NoMask;

            if(configParams._maskType == Selection) {
                ASRLog.e(TAG, String.format(Locale.US, "MaskType : %s", configParams._maskType));
                reader.setMaskType(configParams._maskType);
                ASRLog.e(TAG, String.format(Locale.US, "SL : %s", configParams._sl));
                reader.setQuerySL(configParams._sl);
                ASRLog.e(TAG, String.format(Locale.US, "Session : %s", configParams._session));
                reader.setQuerySession(configParams._session);
                ASRLog.e(TAG, String.format(Locale.US, "Target : %s", configParams._target));
                reader.setQueryTarget(configParams._target);

                for (int i = 0; i < 8; i++) {
                    if (configParams._selectMasks.containsKey(i)) {
                        ASRLog.e(TAG, String.format(Locale.US, "%02d - %s", i, configParams._selectMasks.get(i)._param));
                        reader.setSelectionMask(i, configParams._selectMasks.get(i)._param);
                    } else
                        reader.removeSelectionMask(i);
                }
            }

            if(configParams._maskType == EPC) {
                reader.clearEpcMask();
                for (int i = 0; i < configParams._epcMasks.size(); i++) {
                    reader.addEpcMask(configParams._epcMasks.get(i));
                }
                reader.setEpcMaskMatchMode(configParams._epcNotMatching);
            }
            reader.setReportRSSI(configParams._reportRssi);

            return true;
        } catch(AsReaderException ae) {
            ASRLog.e(TAG, String.format(Locale.US, "SaveToDevice - %s", ae));
            return false;
        }
    }

    public void resetContents() {
        _configParams._selectMasks.clear();
        _configParams._epcMasks.clear();
        _configParams._epcNotMatching = false;
        _configParams._sl = QuerySL.All;
        _configParams._session = QuerySession.S0;
        _configParams._target = QueryTarget.AB;
        _configParams._maskType = NoMask;
        _configParams._reportRssi = false;
        _configParams._displayPc = true;

        _configParams._decodeRescanTime = 0;
        _configParams._powerGain = 300;
        _configParams._operationTime = 0;
        _configParams._linkProfile = 3;
        _configParams._threshold = 50;
        _configParams._inventoryTime = 400;
        _configParams._idleTime = 100;
        _configParams._autoOffTime = 600;
        _configParams._sleepTime = 300;
        _configParams._qValue = 4;
        _configParams._qMin = 0;
        _configParams._qMax = 15;
        _configParams._lbtChannel = null;
    }

    public QuerySL getSL() {
        return _configParams._sl;
    }
    public void setSL(QuerySL value) {
        _configParams._sl = value;
    }
    public QuerySession getSession() {
        return _configParams._session;
    }
    public void setSession(QuerySession value) {
        _configParams._session = value;
    }
    public QueryTarget getTarget() {
        return _configParams._target;
    }
    public void setTarget(QueryTarget value) {
        _configParams._target = value;
    }
    public SelectMaskParam getSelectMask(int index) {
        if (index < _configParams._selectMasks.size()){
            return _configParams._selectMasks.get(index)._param;
        }else {
            return null;
        }
    }
    public void setSelectMask(int index, SelectMaskParam value) {
        _configParams._selectMasks.put(index, new SelectMask(true, value));
    }
    public void removeSelectMask(int index) {
        if(!_configParams._selectMasks.containsKey(index))
            return;
        _configParams._selectMasks.remove(index);
    }
    public void clearSelectMask() {
        _configParams._selectMasks.clear();
    }
    public boolean usedSelectMask(int index) {
        if(!_configParams._selectMasks.containsKey(index))
            return false;
        return _configParams._selectMasks.get(index)._used;
    }

    public void clearEpcMasks() {
        _configParams._epcMasks.clear();
    }
    public void addEpcMask(SelectMaskEpcParam value) {
        _configParams._epcMasks.add(value);
    }
    public SelectMaskEpcParam getEpcMask(int index) {
        return _configParams._epcMasks.get(index);
    }
    public boolean getEpcNotMatching() {
        return _configParams._epcNotMatching;
    }
    public void setEpcNotMatching(boolean value) {
        _configParams._epcNotMatching = value;
    }
    public int getEpcMaskCount() {
        return _configParams._epcMasks.size();
    }
    public MaskType getMaskType() {
        return _configParams._maskType;
    }
    public void setMaskType(MaskType value) {
        _configParams._maskType = value;
    }
    public boolean getReportRssi() {
        return _configParams._reportRssi;
    }
    public void setReportRssi(boolean value) {
        _configParams._reportRssi = value;
    }
    public boolean getDisplayPc() {
        return _configParams._displayPc;
    }
    public void setDisplayPc(boolean value) {
        _configParams._displayPc = value;
    }
    public int getDecodeRescanTime() {
        return _configParams._decodeRescanTime;
    }
    public void setDecodeRescanTime(int value) {
        _configParams._decodeRescanTime = value;
    }
    public int getPowerGain() {
        return _configParams._powerGain;
    }
    public void setPowerGain(int value) {
        _configParams._powerGain = value;
    }
    public int getOperationTime() {
        return _configParams._operationTime;
    }
    public void setOperationTime(int value) {
        _configParams._operationTime = value;
    }
    public int getLinkProfile() {
        return _configParams._linkProfile;
    }
    public void setLinkProfile(int value) {
        _configParams._linkProfile = value;
    }
    public int getThreshold() {
        return _configParams._threshold;
    }
    public void setThreshold(int value) {
        _configParams._threshold = value;
    }
    public int getInventoryTime() {
            return _configParams._inventoryTime;
    }
    public void setInventoryTime(int value) {
        _configParams._inventoryTime = value;
    }
    public int getIdleTime() {
        return _configParams._idleTime;
    }
    public void setIdleTime(int value) {
        _configParams._idleTime = value;
    }
    public int getAutoOffTime() {
        return _configParams._autoOffTime;
    }
    public void setAutoOffTime(int value) {
        _configParams._autoOffTime = value;
    }
    public int getSleepTime() {
        return _configParams._sleepTime;
    }
    public void setSleepTime(int value) {
        _configParams._sleepTime = value;
    }
    public int getQValue() {
        return _configParams._qValue;
    }
    public void setQValue(int value) {
        _configParams._qValue = value;
    }
    public int getQMax() {
        return _configParams._qMax;
    }
    public void setQMax(int value) {
        _configParams._qMax = value;
    }
    public int getQMin() {
        return _configParams._qMin;
    }
    public void setQMin(int value) {
        _configParams._qMin = value;
    }
    public LbtItem[] getLbtChannel() {
        return _configParams._lbtChannel;
    }
    public void setLbtChannel(LbtItem[] value) {
        _configParams._lbtChannel = value;
    }

    private class ConfigParams {
        private HashMap<Integer, SelectMask> _selectMasks = null;
        private ArrayList<SelectMaskEpcParam> _epcMasks = null;
        private QuerySL _sl = QuerySL.All;
        private QuerySession _session = QuerySession.S0;
        private QueryTarget _target = QueryTarget.AB;
        private boolean _epcNotMatching = false;
        private MaskType _maskType = NoMask;
        private boolean _reportRssi = false;
        private boolean _displayPc = true;

        private int _decodeRescanTime = 0;
        private int _powerGain = 0;
        private int _operationTime = 0;
        private int _linkProfile = 0;
        private int _threshold = 0;
        private int _inventoryTime = 0;
        private int _idleTime = 0;
        private int _autoOffTime = 0;
        private int _sleepTime = 0;
        private int _qValue = 0;
        private int _qMax = 0;
        private int _qMin = 0;
        private LbtItem[] _lbtChannel = null;

        private ConfigParams() {
            _selectMasks = new HashMap<Integer, SelectMask>();
            _epcMasks = new ArrayList<SelectMaskEpcParam>();
            _sl = QuerySL.All;
            _session = QuerySession.S0;
            _target = QueryTarget.AB;
            _epcNotMatching = false;
            _maskType = NoMask;
            _reportRssi = false;
            _displayPc = true;

            _decodeRescanTime = 0;
            _powerGain = 300;
            _operationTime = 0;
            _linkProfile = 3;
            _threshold = 50;
            _inventoryTime = 400;
            _idleTime = 100;
            _autoOffTime = 600;
            _sleepTime = 300;
            _qValue = 4;
            _qMin = 0;
            _qMax = 15;
            _lbtChannel = null;
        }
    }

    private class SelectMask {
        private boolean _used = false;
        private SelectMaskParam _param = null;

        private SelectMask(boolean used, SelectMaskParam param) {
            _used = used;
            _param = param;
        }
    }


    /* Recalls the existing mask setting values for mask settings
       when you recall InventoryActivity and OptionActivity. */
    public void setMaskDataUpdate(AsReader reader, MaskType maskType){
        if (_configParams._maskType == null)
            _configParams._maskType = NoMask;


        try {
            reader.clearEpcMask();
            reader.clearSelectionMask();

            switch (maskType){
                case NoMask:
                    break;

                case EPC:
                    SelectMaskEpcParam epcParam;
                    int count = MainActivity.mUserConfigs.getEpcMaskCount();

                    for(int i=0; i<count; i++) {
                        epcParam = MainActivity.mUserConfigs.getEpcMask(i);
                        reader.addEpcMask(epcParam);				// PARAM_EPC_MASK_DATA
                    }

                    final boolean isNotMatching = MainActivity.mUserConfigs.getEpcNotMatching();

                    if (isNotMatching){
                        reader.setEpcMaskMatchMode(true);
                    }else {
                        reader.setEpcMaskMatchMode(false);
                    }

                    break;

                case Selection:
                    SelectMaskParam param;
                    int maskCount = 0;
                    for (int i = AsReader.MIN_SELECTION_MASK; i < AsReader.MAX_SELECTION_MASK;  i++) {

                        if (!MainActivity.mUserConfigs.usedSelectMask(i))
                            break;

                        param = MainActivity.mUserConfigs.getSelectMask(i);

                        if (param == null) {
                            reader.removeSelectionMask(i);		// PARAM_MASK_USED
                        } else {
                            maskCount ++;
                            reader.setSelectionMask(i, param);			//PARAM_MASK
                        }
                    }
                    // If there is a 'Select Mask' setting.
                    if (maskCount > 0)
                        reader.setQuerySL(MainActivity.mUserConfigs.getSL());

                    break;
            }

        } catch (AsReaderException e) {
            e.printStackTrace();
        }

    }
}
