package jp.co.asterisk.asreader.l251g.support.view.base;

import java.util.ArrayList;
import java.util.Locale;

import jp.co.asterisk.asreader.l251g.support.adapter.TagListAdapter;
import jp.co.asterisk.asreader.l251g.support.data.MySQLiteOpenHelper;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.rfm.RFMHelper;
import jp.co.asterisk.asreader.l251g.support.util.ResUtil;

import jp.co.asterisk.asreader.l251g.MainActivity;
import com.asreader.lib.AsReader;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.exception.AsReaderException;
import com.asreader.lib.rfid.type.ActionState;
import com.asreader.lib.rfid.type.CommandType;
import com.asreader.lib.rfid.type.ResultCode;
import com.asreader.lib.rfid.type.MaskType;
import jp.co.asterisk.asreader.l251g.view.ReadMemoryActivity;
import jp.co.asterisk.asreader.l251g.view.TagAccessActivity;
import jp.co.asterisk.asreader.l251g.view.WriteMemoryActivity;
import jp.co.asterisk.asreader.l251g.R;

import android.content.Intent;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public abstract class BaseInventoryActivity extends RfidMaskActivity implements OnCheckedChangeListener {

	private static final String TAG = BaseInventoryActivity.class.getSimpleName();

	protected ListView lstTags;
	protected TextView txtCount;
	protected Button btnInventory;

	protected TagListAdapter adpTags;

	private MenuItem mnuReadMemory;
	private MenuItem mnuWriteMemory;
	private MenuItem mnuTagAccess;

	private boolean mLastContinuousMode = true;
	private boolean mLastReportRssi = false;

	private TextView txtTotalCount;
	private long mTotalTagCount;

	// DB
	protected MySQLiteOpenHelper dbHelper;

	private RFMHelper rfmHelper = RFMHelper.getInstance();

	@Override
	protected void onStart() {
		super.onStart();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onStart()");
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onResume()");
	}

	@Override
	protected void onPause() {
		super.onPause();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onPause()");
	}

	@Override
	protected void onStop() {
		super.onStop();
		insertDB();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onStop()");
	}


	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		if (resultCode == RESULT_DISCONNECTED) {
			setResult(RESULT_DISCONNECTED);
			finish();
			return;
		}

		switch (requestCode) {
		case AccessActivity.VIEW_READ_MEMORY:
		case AccessActivity.VIEW_WRITE_MEMORY:
		case AccessActivity.VIEW_TAG_ACCESS:
			try {
				getReader().setContinuousMode(mLastContinuousMode);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, "onActivityResult - Failed to setContinuousMode");
			}
			try {
				getReader().setReportRSSI(mLastReportRssi);
				MainActivity.mUserConfigs.setReportRssi(mLastReportRssi);
				MainActivity.mUserConfigs.Save();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, "onActivityResult - Failed to setReportRSSI");
			}
			enableActionWidgets(true);
			return;
		}
		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		super.onCreateContextMenu(menu, v, menuInfo);
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateContextMenu()");

		if (v.getId() != R.id.tag_list || getReader().getAction() != ActionState.Stop)
			return;

		getMenuInflater().inflate(R.menu.inventory_menu, menu);

		mnuReadMemory = menu.findItem(R.id.read_memory);
		mnuWriteMemory = menu.findItem(R.id.write_memory);
		mnuTagAccess = menu.findItem(R.id.tag_access);
	}

	@Override
	public boolean onContextItemSelected(MenuItem item) {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onContextItemSelected(%s)", ResUtil.getId(item.getItemId()));

		Intent intent;
		AdapterView.AdapterContextMenuInfo menuInfo = (AdapterView.AdapterContextMenuInfo) item.getMenuInfo();
		int position = menuInfo.position;

		if (position < 0)
			return false;

		mnuReadMemory.setEnabled(false);
		mnuWriteMemory.setEnabled(false);
		mnuTagAccess.setEnabled(false);

		String tag = adpTags.getItem(position, false);
		
		try {
			mLastContinuousMode = getReader().getContinuousMode();
		} catch (AsReaderException e) {
			ASRLog.e(TAG, "onContextItemSelected - Failed to getContinuousMode");
		}
		
		try {
			mLastReportRssi = getReader().getReportRSSI();
		} catch (AsReaderException e) {
			ASRLog.e(TAG, "onContextItemSelected - Failed to getReportRSSI");
		}

		switch (item.getItemId()) {
		case R.id.read_memory:
			intent = new Intent(this, ReadMemoryActivity.class);
			intent.putExtra(KEY_MASK_TYPE, getMaskType().getCode());
			intent.putExtra(KEY_MASK, tag);
			startActivityForResult(intent, AccessActivity.VIEW_READ_MEMORY);
			break;
		case R.id.write_memory:
			intent = new Intent(this, WriteMemoryActivity.class);
			intent.putExtra(KEY_MASK_TYPE, getMaskType().getCode());
			intent.putExtra(KEY_MASK, tag);
			startActivityForResult(intent, AccessActivity.VIEW_WRITE_MEMORY);
			break;
		case R.id.tag_access:
			intent = new Intent(this, TagAccessActivity.class);
			intent.putExtra(KEY_MASK_TYPE, getMaskType().getCode());
			intent.putExtra(KEY_MASK, tag);
			startActivityForResult(intent, AccessActivity.VIEW_TAG_ACCESS);
			break;
		}
		return true;
	}

	@Override
	public void onClick(View v) {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

		switch (v.getId()) {
		case R.id.inventory:
			if (getReader().getAction() == ActionState.Stop) {
				startInventory();
			} else {
				stopInventory();
			}
			break;
		}
		
		if(v.getId() == R.id.mask && mMaskType == MaskType.NoMask) {
			Toast.makeText(this, "Please set the Mask Type in RFID Option menu before using mask.", Toast.LENGTH_SHORT).show();
			return;
		}
		
		super.onClick(v);
	}

	@Override
	public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCheckedChanged(%s, %s)", ResUtil.getId(buttonView.getId()), isChecked);
	}

	@Override
	public void onActionChanged(AsReader reader, ActionState action) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onActionChanged(%s)", action);

		adpTags.notifyDataSetChanged();

		enableActionWidgets(true);
	}

	@Override
	public void onCommandComplete(AsReader reader, CommandType command) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onCommandComplete(%s)", command);
		
		WaitDialog.hide();
		enableActionWidgets(true);
	}

	@Override
	public void onReadTag(AsReader reader, ActionState action, String tag, float rssi, float phase, float frequency) {
		mTotalTagCount++;
		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCount));

		adpTags.addTag(tag, rssi, phase, frequency, System.currentTimeMillis());
		txtCount.setText(String.format(Locale.US, "%d", adpTags.getCount()));

		ASRLog.d(TAG, ASRLog.DEBUG, "EVENT. onReadTag(%s, [%s], %.2f, %.2f)", action, tag, rssi, phase);
	}

	@Override
	public void onReadTagWithRFM(AsReader reader, ActionState action, String tag, float rssi, float phase, float frequency, String[] rfm_data) {
		mTotalTagCount++;
		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCount));

		// rfm_data : tid, temperatureCode, calibration, sensorCode
		String temp = rfmHelper.calculationTemp(rfm_data[1], rfm_data[2]);
		String sensorcode = rfmHelper.calculationSensorCode(rfm_data[3],rfm_data[0]);

		adpTags.addTag(tag,
				rfm_data[0]==null?"null":rfm_data[0],
				rssi,
				phase,
				frequency,
				System.currentTimeMillis(),
				rfm_data[1]==null?"null":rfm_data[1],
				rfm_data[3]==null?"null":rfm_data[3],
				temp==null?"null":temp,
				sensorcode==null?"null":sensorcode,
				rfm_data[2]==null?"null":rfm_data[2]);
		txtCount.setText(String.format(Locale.US, "%d", adpTags.getCount()));
	}

	@Override
	public void onAccessResult(AsReader reader, ResultCode code, ActionState action, String epc, String data,
			float rssi, float phase, float frequency) {

		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onAccessResult(%s, [%s], [%s]%.2f, %.2f)", action, epc, data, rssi, phase);
		
		adpTags.addTag(epc, data, rssi, phase, frequency, System.currentTimeMillis());
		txtCount.setText(String.format(Locale.US, "%d", adpTags.getCount()));
	}

	// Initialize Widgets
	@Override
	protected void initWidgets() {
		super.initWidgets();

		// Total Count TextView
		txtTotalCount = (TextView) findViewById(R.id.total_tag_count);
		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCount));

		mTotalTagCount = 0;

		btnOption = (Button) findViewById(R.id.option);
		if(btnOption != null)
			btnOption.setOnClickListener(this);

		// Tag ListView
		lstTags = (ListView) findViewById(R.id.tag_list);
		adpTags = new TagListAdapter(this);

		// new database
		dbHelper = MySQLiteOpenHelper.getInstance(this);

		// Count TextView
		txtCount = (TextView) findViewById(R.id.tag_count);
		txtCount.setText(String.format(Locale.US, "%d", adpTags.getCount()));
		// Inventory Button
		btnInventory = (Button) findViewById(R.id.inventory);
		btnInventory.setOnClickListener(this);

		getDB();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. initWidgets()");
	}

	// Enable/Disable Action Widgets
	@Override
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);

		btnMask.setEnabled(isEnabledWidget(enabled) && mMaskType != MaskType.NoMask);
		lstTags.setEnabled(isEnabledWidget(enabled));
		enableActionButton(btnInventory, enabled, R.string.action_inventory, R.string.action_stop);
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}
	
	@Override
	protected void clearWidgets() {

		// DB Delete
		dbHelper.delete();

		mTotalTagCount = 0;
		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCount));

		adpTags.clear();
		txtCount.setText(String.format(Locale.US, "%d", adpTags.getCount()));

		ASRLog.i(TAG, ASRLog.INFO, "INFO. clearWidgets()");
	}

	protected void startInventory() {
		enableActionWidgets(false);
		ResultCode res;
		if ((res = getReader().inventory()) != ResultCode.NoError) {

			ASRLog.e(TAG, "ERROR. startInventory() - Failed to start inventory()");
			String msg = String.format(Locale.US, "%s : 0x%04X", res.toString(), res.getCode());
			Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
			enableActionWidgets(true);
			return;
		}

		ASRLog.i(TAG, ASRLog.INFO, "INFO. startInventory()");
	}

	protected void stopInventory() {

		enableActionWidgets(false);
		if (getReader().stop() != ResultCode.NoError) {
			ASRLog.e(TAG, "ERROR. stopInventory() - Failed to stop inventory()");
			enableActionWidgets(true);
			insertDB();
			return;
		}
		ASRLog.i(TAG, ASRLog.INFO, "INFO. stopInventory()");
	}

	public void setReportRssiMode(boolean enabled) {
		// 1) Save current data list to DB.
		insertDB();

		this.adpTags.setReportRSSI(enabled);

		// 2) on the list of currently cleared get DB data.
		getDB();
	}
	
	public void setDisplayPC(boolean enabled) {
		this.adpTags.setDisplayPc(enabled);
	}

	// DB
	protected void getDB() {

		// data from db
		ArrayList dbTag = dbHelper.select();

		for (int i=0; i<dbTag.size(); i++){
			if (dbTag.get(i).equals(""))
				break;

			String[] temp = dbTag.get(i).toString().split(",");
//			Log.d(TAG, "# DB get : "+ dbTag.get(i));

			adpTags.addDBTag(temp[0], Integer.parseInt(temp[1]),Float.parseFloat(temp[2]),Float.parseFloat(temp[3]),Float.parseFloat(temp[4]),Long.parseLong(temp[5]));
			mTotalTagCount = Long.parseLong(temp[6]);
		}

		lstTags.setAdapter(adpTags);
		adpTags.start();

		registerForContextMenu(lstTags);
		txtCount.setText(String.format(Locale.US, "%d", adpTags.getCount()));
		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCount));
	}

	protected void insertDB() {
		// DB data delete.
		dbHelper.delete();

		// DB insert
		for (int i =0; i<adpTags.getCount(); i++){
//			Log.d(TAG,"# DB stop : set DB List item : " + adpTags.getItemForDB(i));

			String[] temp = adpTags.getItemForDB(i).split(",");
			dbHelper.insert(temp[0],temp[1],temp[2],temp[3],temp[4],mTotalTagCount,temp[5]);
		}
	}

}
