package jp.co.asterisk.asreader.l251g.support.view.base;

import java.util.Locale;

import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.util.ResUtil;

import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.type.MemoryBank;
import jp.co.asterisk.asreader.l251g.R;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

public abstract class MemoryActivity extends AccessActivity {

	private static final String TAG = MemoryActivity.class.getSimpleName();

	@SuppressWarnings("unused")
	protected static final int MAX_MEM_OFFSET = 16;
	protected static final int DEFAULT_MEM_OFFSET = 2;

	private TextView txtBank;
	private TextView txtOffset;

	private MemoryBank mBank;
	private int mOffset;

	@Override
	protected void initWidgets() {
		super.initWidgets();

		// Bank Spinner
		txtBank = (TextView) findViewById(R.id.bank);
		txtBank.setOnClickListener(this);

		// Offset Spinner
		txtOffset = (TextView) findViewById(R.id.offset);
		txtOffset.setOnClickListener(this);

		setMemBank(MemoryBank.EPC);
		setMemOffset(DEFAULT_MEM_OFFSET);

		ASRLog.i(TAG, ASRLog.INFO, "INFO. initWidgets()");
	}

	@Override
	public void onClick(View v) {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

		switch (v.getId()) {
		case R.id.bank:
			showMemBankDialog();
			break;
		case R.id.offset:
			showMemOffsetDialog();
			break;
		}
		super.onClick(v);
	}

	@Override
	protected void clearWidgets() {
		super.clearWidgets();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. clearWidgets()");
	}

	protected void setMemBank(MemoryBank bank) {
		mBank = bank;
		txtBank.setText(mBank.toString());
		ASRLog.i(TAG, ASRLog.INFO, "INFO. setMemBank(%s)", bank);
	}

	protected MemoryBank getMemBank() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. getMemBank() - [%s]", mBank);
		return mBank;
	}

	protected void setMemOffset(int offset) {
		mOffset = offset;
		txtOffset.setText(String.format(Locale.US, "%d WORD", offset));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. setMemOffset(%d)", offset);
	}

	protected int getMemOffset() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. getMemOffset() - [%d]", mOffset);
		return mOffset;
	}

	private void showMemBankDialog() {

		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView bank = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for (MemoryBank item : MemoryBank.values()) {
			adapter.addItem(item.getValue(), item.toString());
		}
		bank.setAdapter(adapter);
		bank.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.bank);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = bank.getCheckedItemPosition();
				MemoryBank value = MemoryBank.valueOf(adapter.getValue(position));
				setMemBank(value);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMemBankDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMemBankDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {
			
			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMemBankDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {
			
			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mBank.getValue());
				bank.setItemChecked(position, true);
				bank.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMemBankDialog().onShow()");
			}
		});
		dialog.show();
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMemBankDialog()");
	}

	private void showMemOffsetDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);

		for(int n=0; n<=15; n++) {
			adapter.addItem(n, String.format(Locale.US, "%d WORD", n));
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.offset);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position;
				try {
					position = value.getCheckedItemPosition();
				} catch (Exception e) {
					position = DEFAULT_MEM_OFFSET;
				}
				setMemOffset(position);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMemOffsetDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMemOffsetDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMemOffsetDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mOffset);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMemOffsetDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMemOffsetDialog()");
	}

}
