package jp.co.asterisk.asreader.l251g.support.view.base;

import jp.co.asterisk.asreader.l251g.support.util.ResUtil;

import jp.co.asterisk.asreader.l251g.R;
import com.asreader.lib.barcode.BarcodeType;
import com.asreader.lib.AsReaderManager;
import com.asreader.lib.AsReader;
import com.asreader.lib.device.type.ConnectionState;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.event.AsReaderEventListener;
import com.asreader.lib.rfid.type.ActionState;
import com.asreader.lib.rfid.type.CommandType;
import com.asreader.lib.device.type.KeyState;
import com.asreader.lib.device.type.KeyType;
import com.asreader.lib.rfid.type.ResultCode;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.widget.Button;

public abstract class RfidActivity extends SubActivity implements AsReaderEventListener {

	private static final String TAG = RfidActivity.class.getSimpleName();

	protected static final int VIEW_MASK = 1000;
	protected static final int VIEW_BARCODE_OPTION_GS1_COMPOSITE = 2001;
	protected static final int VIEW_BARCODE_OPTION_OTHERS = 2010;

	protected static final int RESULT_DISCONNECTED = Activity.RESULT_FIRST_USER + 1;

	public static final String KEY_ACTION = "key_action";
	
	public static final boolean DEFAULT_KEY_ACTION = true;

	private AsReader mReader = null;
	private boolean mKeyAction;

	private boolean mIsAlert =true;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	
		mKeyAction = getIntent().getBooleanExtra(KEY_ACTION, DEFAULT_KEY_ACTION);
	}

	// Retrieve RFID Reader
	protected AsReader getReader() {
		return mReader;
	}

	// Create Reader
	protected void createReader() {
		mReader = AsReaderManager.getInstance();
		mReader.setEventListener(this);

		ASRLog.i(TAG, ASRLog.INFO, "INFO. createReader()");
	}

	@Override
	protected void onPause() {
		super.onPause();
		if (isFinishing()) {
			mReader.removeEventListener(this);
		}
	}

	// Destroy Reader
	protected void destroyReader() {
//		mReader.removeEventListener(this);
		// Destroy Rfid Reader
		AsReaderManager.onDestroy();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. destroyReader()");
	}

	// Initialize Reader
	protected abstract void initReader();

	protected boolean isReaderAction() {
		return mReader.getAction() != ActionState.Stop;
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		if ((requestCode == VIEW_MASK || requestCode == VIEW_BARCODE_OPTION_GS1_COMPOSITE || requestCode == VIEW_BARCODE_OPTION_OTHERS) &&
				resultCode == RESULT_DISCONNECTED) {
			setResult(RESULT_DISCONNECTED);
			finish();
			return;
		}

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public void onStateChanged(AsReader reader, ConnectionState state) {
		ASRLog.d(TAG, ASRLog.DEBUG, "EVENT. onStateChanged(%s)", state);

		if (state == ConnectionState.Disconnected) {
			setResult(RESULT_DISCONNECTED);
			finish();
		}

		// SDK sends a message about an error or sleep event.
		if (state == ConnectionState.SDKEventMessage){
			new Thread(new Runnable() {
				@Override
				public void run() {
					runOnUiThread(new Runnable(){
						@Override
						public void run() {
							if (mIsAlert){
								showDialog(mReader.getSDKMessage());
								mIsAlert = false;
							}
						}
					});
				}
			}).start();
		}
	}


	@Override
	public void onActionChanged(AsReader reader, ActionState action) {
		ASRLog.d(TAG, ASRLog.DEBUG, "EVENT. onActionChanged(%s)", action);
	}

	@Override
	public void onCommandComplete(AsReader reader, CommandType command) {
		ASRLog.d(TAG, ASRLog.DEBUG, "EVENT. onCommandComplete(%s)", command);
	}

	@Override
	public void onReadTag(AsReader reader, ActionState action, String tag, float rssi, float phase, float frequency) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onReadedTag(%s, [%s], %.2f, %.2f)", action, tag, rssi, phase);
	}

	@Override
	public void onReadTagWithRFM(AsReader reader, ActionState action, String tag, float rssi, float phase, float frequency, String[] rfm_data) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onReadedRFMTag(%s, [%s], %.2f, %.2f)", action, tag, rssi, phase);
	}

	@Override
	public void onAccessResult(AsReader reader, ResultCode code, ActionState action, String epc, String data,
			float rssi, float phase, float frequency) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onAccessResult(%s, %s, [%s], [%s], %.2f, %.2f)", code, action, epc, data, rssi, phase);
	}

	@Override
	public void onReadBarcode(AsReader reader, BarcodeType type, String codeId, String barcode) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onDetactBarcode (%s, %s, [%s])", type, codeId, barcode);
	}

	@Override
	public void onExceptionOccurred(AsReader reader, int code) {
		ASRLog.e(TAG, "DEBUG. onExceptionOccurred : 0x%X", code);
	}

	@Override
	public void onKeyEvent(AsReader reader, KeyType key, KeyState state) {
			
	}

	@Override
	protected void initWidgets() {
		
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		
	}

	protected void enableActionButton(Button action, boolean enabled, int actionId, int stopId) {
		action.setEnabled(enabled);
		
		if (enabled) {
			action.setText(getReader().getAction() == ActionState.Stop ? actionId : stopId);
		}
		ASRLog.i(TAG, ASRLog.INFO, "INFO. enableActionButton(%s, %s, %d, %d)", ResUtil.getId(action.getId()), enabled, actionId,
				stopId);
	}

	private void showDialog(String msg) {

		if (this.isFinishing())
			return;

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.basic_alert);
		builder.setMessage(msg);
		builder.setCancelable(false);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				mIsAlert = true;
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showDialog().$PositiveButton.onClick()");
			}
		});

		AlertDialog dialog = builder.create();

		if (!dialog.isShowing())
			dialog.show();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. showDialog()");
	}

	protected boolean isEnabledWidget(boolean enabled) {
		ActionState action = getReader().getAction();
		return enabled ? (action == ActionState.Stop ? enabled : false) : enabled;
	}

	protected boolean isEnabledWidget(boolean enabled, ActionState state) {
		ActionState action = getReader().getAction();
		return enabled ? (action == ActionState.Stop ? enabled : (action == state ? enabled : false)) : enabled;
	}

	protected String getResultCode(int resultCode) {

		switch (resultCode) {
		case RESULT_DISCONNECTED:
			return "RESULT_DISCONNECTED";
		}
		return super.getResultCode(resultCode);
	}
	
	protected boolean getKeyAction() {
		return mKeyAction;
	}
	
	protected void setKeyAction(boolean enabled) {
		mKeyAction = enabled;
	}
	
	@SuppressLint("NewApi")
	@SuppressWarnings("deprecation")
	protected int getColor(Context context, int id) {
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			return context.getColor(id);
		else
			return context.getResources().getColor(id);
	}
}
