package jp.co.asterisk.asreader.l251g.support.view.base;

import java.util.Locale;

import jp.co.asterisk.asreader.l251g.support.util.ResUtil;

import com.asreader.lib.barcode.BarcodeType;
import com.asreader.lib.diagnostics.ASRLog;

import android.app.Activity;
import android.view.MenuItem;

public abstract class SubActivity extends Activity {

	private static final String TAG = SubActivity.class.getSimpleName();
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onOptionsItemSelected(%s)", ResUtil.getId(item.getItemId()));
		
		switch (item.getItemId()) {
		case android.R.id.home:
			this.setResult(Activity.RESULT_CANCELED);
			this.finish();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	protected abstract void initWidgets();

	protected abstract void enableActionWidgets(boolean enabled);

	protected String getRequestCode(int requestCode) {
		return String.format(Locale.US, "%d", requestCode);
	}

	protected String getResultCode(int resultCode) {
		switch (resultCode) {
		case Activity.RESULT_CANCELED:
			return "RESULT_CANCELED";
		case Activity.RESULT_OK:
			return "RESULT_OK";
		case Activity.RESULT_FIRST_USER:
			return "RESULT_FIRST_USER";
		}
		return String.format(Locale.US, "%d", resultCode);
	}

	protected BarcodeType getBarcodeType(String asciiValue) {

		char[] chars = asciiValue.toCharArray();
		StringBuffer hex = new StringBuffer();
		for (int i = 0; i < chars.length; i++)
		{
			hex.append(Integer.toHexString((int) chars[i]));
		}

		byte type = (byte) Integer.parseInt(hex.toString(),16);

		switch ((int)type) {
			case 0x41:
				return BarcodeType.AustralianPost;
			case 0x7A:
				return BarcodeType.AztecCode;
			case 0x42:
				return BarcodeType.BritishPost;
			case 0x43:
				return BarcodeType.CanadianPost;
			case 0x51:
				return BarcodeType.ChinaPost;
			case 0x61:
				return BarcodeType.Codabar;
			case 0x71:
				return BarcodeType.CodablockF;
			case 0x68:
				return BarcodeType.Code11;
			case 0x6A:
				return BarcodeType.Code128;
			case 0x6F:
				return BarcodeType.Code16K; // not supported
			case 0x3C:
				return BarcodeType.Code32;
			case 0x62:
				return BarcodeType.Code39;
			case 0x6C:
				return BarcodeType.Code49;	// not supported
			case 0x69:
				return BarcodeType.Code93;
			case 0x77:
				return BarcodeType.DataMatrix;
			case 0x64:
				return BarcodeType.EAN13;
			case 0x44:
				return BarcodeType.EAN8;
			case 0x65:
				return BarcodeType.I2of5;
			case 0x4A:
				return BarcodeType.JapanesePost;
			case 0x4B:
				return BarcodeType.KixPost;
			case 0x3F:
				return BarcodeType.KoreaPost;
			case 0x6D:
				return BarcodeType.X2of5;
			case 0x78:
				return BarcodeType.MaxiCode;
			case 0x52:
				return BarcodeType.MicroPDF;
			case 0x67:
				return BarcodeType.MSI;
			case 0x4F:
				return BarcodeType.OCR;	// not supported
			case 0x72:
				return BarcodeType.PDF417;
			case 0x4C:
				return BarcodeType.PlanetCode;
			case 0x6E:
				return BarcodeType.PlesseyCode;	// not supported
			case 0x57:
				return BarcodeType.PosiCode;	// not supported
			case 0x50:
				return BarcodeType.Postnet;
			case 0x73:
				return BarcodeType.QRCode;
			case 0x79:
				return BarcodeType.GS1DataBar;
			case 0x7B:
				return BarcodeType.GS1DataBarLimited;
			case 0x7D:
				return BarcodeType.GS1DataBarExpanded;
			case 0x66:
				return BarcodeType.R2of5;
			case 0x54:
				return BarcodeType.TLC39;
			case 0x74:
				return BarcodeType.Telepen;
			case 0x3D:
				return BarcodeType.TriopticCode;	// not supported
			case 0x49:
				return BarcodeType.GS1128;
			case 0x63:
				return BarcodeType.UPCA;
			case 0x45:
				return BarcodeType.UPCE;
			case 0x59:
				return BarcodeType.NEC2of5;
			case 0x48:
				return BarcodeType.ChineseSensible;
			case 0x56:
				return BarcodeType.CodablockA;
			case 0x2C:
				return BarcodeType.InfoMail;
			case 0x4D:
				return BarcodeType.IntelligentMailBarCode;
			case 0x4E:
				return BarcodeType.Postal4i;
			case 0x9C:
			default:
				return BarcodeType.NoRead;
		}
	}
}
