package jp.co.asterisk.asreader.l251g.support.view.sub;

import jp.co.asterisk.asreader.l251g.support.adapter.SelectMaskEpcAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.IDialogResultListener;
import jp.co.asterisk.asreader.l251g.support.dialog.SelectMaskEpcDialog;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.util.ResUtil;

import jp.co.asterisk.asreader.l251g.MainActivity;
import com.asreader.lib.barcode.BarcodeType;
import com.asreader.lib.device.type.ConnectionState;
import com.asreader.lib.device.type.KeyState;
import com.asreader.lib.device.type.KeyType;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.event.AsReaderEventListener;
import com.asreader.lib.rfid.exception.AsReaderException;
import com.asreader.lib.rfid.params.SelectMaskEpcParam;
import com.asreader.lib.rfid.type.ActionState;
import com.asreader.lib.rfid.type.CommandType;
import com.asreader.lib.rfid.type.ResultCode;
import jp.co.asterisk.asreader.l251g.R;
import com.asreader.lib.AsReaderManager;
import com.asreader.lib.AsReader;

import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.ListView;
import android.widget.TextView;

import java.util.Locale;

public class EpcMaskActivity extends Activity implements AsReaderEventListener, OnClickListener, IDialogResultListener,
		OnItemClickListener, android.content.DialogInterface.OnClickListener {

	private static final String TAG = EpcMaskActivity.class.getSimpleName();
	
	private static final int RESULT_DISCONNECTED = Activity.RESULT_FIRST_USER + 1;
	private static final int REMOVE_QUESTION_DIALOG = 2003;
	private static final int REMOVE_ALL_QUESTION_DIALOG = 2103;

	private AsReader mReader = null;

	private ListView lstMasks;
	private CheckBox chkNotMatching;
	private TextView txtCount;
	private Button btnMaskAdd;
	private Button btnMaskEdit;
	private Button btnMaskRemove;
	private Button btnMaskClear;
	private Button btnSave;
	private Button btnCancel;

	private SelectMaskEpcAdapter adpMasks;

	private SelectMaskEpcDialog mDialog;

	private Handler mEvent;

	//private UserMasks _userMasks;

	// ************************************************************************
	// Activity Event Methods
	// ************************************************************************

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_selection_mask_epc);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayHomeAsUpEnabled(true);
		}

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		//_userMasks = new UserMasks(this.getApplicationContext());


		// Get RFID Reader Instance
		mReader = AsReaderManager.getInstance();
		mReader.setEventListener(this);

		// Initialize Widget
		initWidgets();
		// Enable action widget
		enableActionWidgets(false);

		mEvent = new Handler();

		// Load Selection Mask
		WaitDialog.show(this, R.string.load_selection_mask);
		new Thread(new Runnable() {

			@Override
			public void run() {
				//_userMasks.Load();
				try {
					SelectMaskEpcParam item;
					//int count = mReader.getEpcMaskCount();
					int count = MainActivity.mUserConfigs.getEpcMaskCount();

					for (int i = 0; i < count; i++) {
						//item = mReader.getEpcMask(i);
						item = MainActivity.mUserConfigs.getEpcMask(i);
						adpMasks.addItem(item);
					}
					//final boolean isMatchMode = mReader.getEpcMaskMatchMode();
					final boolean isNotMatching = MainActivity.mUserConfigs.getEpcNotMatching();
					mEvent.post(new Runnable() {

						@Override
						public void run() {
							chkNotMatching.setChecked(isNotMatching);
							adpMasks.notifyDataSetChanged();
							updateMaskCount();
							enableActionWidgets(true);
							WaitDialog.hide();
						}

					});
				} catch (Exception e) {
					mEvent.post(new Runnable() {

						@Override
						public void run() {
							chkNotMatching.setChecked(false);
							adpMasks.notifyDataSetChanged();
							updateMaskCount();
							enableActionWidgets(true);
							WaitDialog.hide();
						}

					});
					ASRLog.e(TAG, e, "ERROR. onCreate().$Runnable.run() - Failed to load select mask EPC");
				}
			}

		}).start();
	}

	@Override
	protected void onDestroy() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		mReader.removeEventListener(this);
		// Destroy RFID Reader
		AsReaderManager.onDestroy();

		super.onDestroy();
	}


	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onOptionsItemSelected(home)");
			setResult(Activity.RESULT_CANCELED);
			finish();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@SuppressWarnings("deprecation")
	@Override
	public void onClick(View v) {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

		switch (v.getId()) {
		case R.id.add_mask:
			showDialog(SelectMaskEpcDialog.NEW_ITEM);
			break;
		case R.id.edit_mask:
			showDialog(SelectMaskEpcDialog.MODIFY_ITEM);
			break;
		case R.id.remove_mask:
			showDialog(REMOVE_QUESTION_DIALOG);
			break;
		case R.id.clear_mask:
			showDialog(REMOVE_ALL_QUESTION_DIALOG);

			break;
		case R.id.save:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.save_selection_mask);
			final boolean isEpcNotMatching = chkNotMatching.isChecked();
			new Thread(new Runnable() {

				@Override
				public void run() {
					try {
						ASRLog.i(TAG, ASRLog.INFO, "+++ INFO. onClick(save)$Runnable.run()");
						SelectMaskEpcParam item;
						int count = adpMasks.getCount();
						mReader.clearEpcMask();
						MainActivity.mUserConfigs.clearEpcMasks();
						for (int i = 0; i < count; i++) {
							item = adpMasks.getItem(i);
							mReader.addEpcMask(item);
							MainActivity.mUserConfigs.addEpcMask(item);
						}
						mReader.setEpcMaskMatchMode(isEpcNotMatching);
						MainActivity.mUserConfigs.setEpcNotMatching(isEpcNotMatching);
						MainActivity.mUserConfigs.Save();

						mEvent.post(new Runnable() {

							@Override
							public void run() {
								ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(save)$Runnable.run()$Runable.run()");
								WaitDialog.hide();
								finish();
							}

						});

					} catch (AsReaderException e) {
						ASRLog.e(TAG, e, "--- ERROR. onClick(save)$Runnable.run() - Failed to save select mask EPC");
						return;
					}
					ASRLog.i(TAG, ASRLog.INFO, "--- INFO. onClick(save)$Runnable.run()");
				}

			}).start();
			break;
		case R.id.cancel:
			finish();
			break;
		}
	}

	@Override
	public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onItemClick(mask_list, %d, %d)", position, id);
		enableActionWidgets(true);
	}

	// ************************************************************************
	// Select Mask EPC Dialog Interface Methods
	// ************************************************************************

	@Override
	protected Dialog onCreateDialog(int id) {
		switch (id) {
		case SelectMaskEpcDialog.NEW_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateDialog(NEW_ITEM)");
			if (mDialog == null) {
				mDialog = new SelectMaskEpcDialog(this);
				mDialog.setResultListener(this);
			}
			return mDialog.getDialog();
		case SelectMaskEpcDialog.MODIFY_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateDialog(MODIFY_ITEM)");
			if (mDialog == null) {
				mDialog = new SelectMaskEpcDialog(this);
				mDialog.setResultListener(this);
			}
			return mDialog.getDialog();
		case REMOVE_QUESTION_DIALOG:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateDialog(REMOVE_QUESTION_DIALOG)");
			AlertDialog.Builder builder = new AlertDialog.Builder(this);
			builder.setIcon(android.R.drawable.ic_dialog_alert);
			builder.setTitle(R.string.action_remove);
			builder.setMessage(R.string.remove_mask_message);
			builder.setPositiveButton(R.string.action_yes, new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog, int which) {
					int position = lstMasks.getCheckedItemPosition();
					if (position < 0)
						return;
					lstMasks.setItemChecked(position, false);
					adpMasks.removeItem(position);
					updateMaskCount();
					enableActionWidgets(true);
				}
			});
			builder.setNegativeButton(R.string.action_no, null);
			return builder.create();
		case REMOVE_ALL_QUESTION_DIALOG:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateDialog(REMOVE_ALL_QUESTION_DIALOG)");
			AlertDialog.Builder dlg = new AlertDialog.Builder(this);
			dlg.setIcon(android.R.drawable.ic_dialog_alert);
			dlg.setTitle(R.string.action_remove_all);
			dlg.setMessage(R.string.remove_all_mask_message);
			dlg.setPositiveButton(R.string.action_yes, new DialogInterface.OnClickListener() {
				@Override
				public void onClick(DialogInterface dialog, int which) {
					int position = lstMasks.getCheckedItemPosition();
					if (position >= 0) {
						lstMasks.setItemChecked(position, false);
					}
					adpMasks.clear();
					enableActionWidgets(true);
				}
			});
			dlg.setNegativeButton(R.string.action_no, null);
			return dlg.create();
		}
		return null;
	}

	@Override
	protected void onPrepareDialog(int id, Dialog dialog) {
		switch (id) {
		case SelectMaskEpcDialog.NEW_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onPrepareDialog(NEW_ITEM)");
			mDialog.setState(SelectMaskEpcDialog.NEW_ITEM);
			mDialog.clearItem();
			break;
		case SelectMaskEpcDialog.MODIFY_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onPrepareDialog(MODIFY_ITEM)");
			mDialog.setState(SelectMaskEpcDialog.MODIFY_ITEM);
			int position = lstMasks.getCheckedItemPosition();
			SelectMaskEpcParam item = adpMasks.getItem(position);
			mDialog.setItem(item);
			break;
		case REMOVE_QUESTION_DIALOG:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onPrepareDialog(REMOVE_QUESTION_DIALOG)");
			break;
		}
	}

	@Override
	public void onOkClick(int what, DialogInterface dialog) {
		SelectMaskEpcParam item;

		switch (what) {
		case SelectMaskEpcDialog.NEW_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onOkClick(NEW_ITEM)");
			item = mDialog.getItem();
			if(item != null) {
				adpMasks.addItem(item);
				updateMaskCount();
			}
			break;
		case SelectMaskEpcDialog.MODIFY_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onOkClick(MODIFY_ITEM)");
			item = mDialog.getItem();
			if(item != null) {
				int position = lstMasks.getCheckedItemPosition();
				adpMasks.updateItem(position, item);
				updateMaskCount();
			}
			break;
		}
		enableActionWidgets(true);
	}

	@Override
	public void onCancelClick(int what, DialogInterface dialog) {
		switch (what) {
		case SelectMaskEpcDialog.NEW_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCancelClick(NEW_ITEM)");
			break;
		case SelectMaskEpcDialog.MODIFY_ITEM:
			ASRLog.i(TAG, ASRLog.INFO, "INFO. onCancelClick(MODIFY_ITEM)");
			break;
		}
		enableActionWidgets(true);
	}

	@Override
	public void onClick(DialogInterface dialog, int which) {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. DialogInterface.onClick()");
	}

	// ************************************************************************
	// RFID Reader Event Methods
	// ************************************************************************
	@Override
	public void onStateChanged(AsReader reader, ConnectionState state) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onStateChanged(%s)", state);
		if (state == ConnectionState.Disconnected) {
			setResult(RESULT_DISCONNECTED);
			finish();
		}
	}

	@Override
	public void onActionChanged(AsReader reader, ActionState action) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onActionChanged(%s)", action);
	}

	@Override
	public void onReadTag(AsReader reader, ActionState action, String tag, float rssi, float phase, float frequency) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onReadedTag(%s, [%s], %.2f, %.2f)", action, tag, rssi, phase);
	}

	@Override
	public void onReadTagWithRFM(AsReader reader, ActionState action, String tag, float rssi, float phase, float frequency, String[] rfm_data) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onReadedRFMTag(%s, [%s], %.2f, %.2f)", action, tag, rssi, phase);
	}

	@Override
	public void onAccessResult(AsReader reader, ResultCode code, ActionState action, String epc, String data,
			float rssi, float phase, float frequency) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onAccessResult(%s, %s, [%s], [%s], %.2f, %.2f)", code, action, epc, data, rssi, phase);
	}

	@Override
	public void onCommandComplete(AsReader reader, CommandType command) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onCommandComplete(%s)", command);
	}

//	@Override
//	public void onDebugMessage(AsReader reader, String msg) {
//		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onDebugMessage([%s])", msg);
//	}

	@Override
	public void onReadBarcode(AsReader reader, BarcodeType type, String codeId, String barcode) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onDetactBarcode(%s, [%s], [%s])", type, codeId, barcode);
	}

//	@Override
//	public void onRemoteKeyStateChanged(AsReader reader, RemoteKeyState state) {
//		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onRemoteKeyStateChanged(%s)", state);
//	}

	// ************************************************************************
	// Helper Methods
	// ************************************************************************

	// Update Mask Count
	private void updateMaskCount() {
		adpMasks.notifyDataSetChanged();
		int count = adpMasks.getCount();
		txtCount.setText(String.format(Locale.US, "%d", count));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. updateMaskCount()");
	}

	// Initialize Widgets
	private void initWidgets() {
		lstMasks = (ListView) findViewById(R.id.mask_list);
		adpMasks = new SelectMaskEpcAdapter(this);
		lstMasks.setAdapter(adpMasks);
		lstMasks.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		lstMasks.setOnItemClickListener(this);

		chkNotMatching = (CheckBox) findViewById(R.id.match_mode);

		txtCount = (TextView) findViewById(R.id.mask_count);

		btnMaskAdd = (Button) findViewById(R.id.add_mask);
		btnMaskAdd.setOnClickListener(this);
		btnMaskEdit = (Button) findViewById(R.id.edit_mask);
		btnMaskEdit.setOnClickListener(this);
		btnMaskRemove = (Button) findViewById(R.id.remove_mask);
		btnMaskRemove.setOnClickListener(this);
		btnMaskClear = (Button) findViewById(R.id.clear_mask);
		btnMaskClear.setOnClickListener(this);

		btnSave = (Button) findViewById(R.id.save);
		btnSave.setOnClickListener(this);

		btnCancel = (Button) findViewById(R.id.cancel);
		btnCancel.setOnClickListener(this);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initWidgets()");
	}

	// Enable/Disable Action Widgets
	private void enableActionWidgets(boolean enabled) {
		int position = lstMasks.getCheckedItemPosition();
		int count = lstMasks.getCount();

		lstMasks.setEnabled(enabled);
		chkNotMatching.setEnabled(enabled);
		btnMaskAdd.setEnabled(enabled);
		btnMaskEdit.setEnabled(enabled && position >= 0);
		btnMaskRemove.setEnabled(enabled && position >= 0);
		btnMaskClear.setEnabled(enabled && count > 0);
		btnSave.setEnabled(enabled);
		btnCancel.setEnabled(enabled);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}

	@Override
	public void onExceptionOccurred(AsReader reader, int code) {
		
	}

	@Override
	public void onKeyEvent(AsReader reader, KeyType key, KeyState state) {
		
	}
}
