package jp.co.asterisk.asreader.l251g.support.view.sub.barcode;


import jp.co.asterisk.asreader.l251g.support.adapter.BarcodeSymbolStateListAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

import com.asreader.lib.barcode.spc.ParamValue;
import com.asreader.lib.barcode.spc.ParamValueList;
import com.asreader.lib.barcode.spc.type.ParamName;
import com.asreader.lib.diagnostics.ASRLog;
import jp.co.asterisk.asreader.l251g.R;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ListView;

public class EnableSymbolActivity extends RfidActivity implements OnClickListener {

	private static final String TAG = EnableSymbolActivity.class.getSimpleName();

	private ListView lstSymbols;
	private Button btnOption;

	private BarcodeSymbolStateListAdapter adpSymbols;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_barcode_option_enable_symbol);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayHomeAsUpEnabled(true);
		}

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		this.createReader();

		// Initialize Widget
		this.initWidgets();

		// Initialize Reader
		this.initReader();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		this.destroyReader();

		super.onDestroy();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.option:
			break;
		}
	}

	@Override
	protected void initWidgets() {
		lstSymbols = (ListView) findViewById(R.id.symbols);
		lstSymbols.setChoiceMode(ListView.CHOICE_MODE_MULTIPLE);
		adpSymbols = new BarcodeSymbolStateListAdapter(this);
		lstSymbols.setAdapter(adpSymbols);
		btnOption = (Button) findViewById(R.id.set_option);
		btnOption.setOnClickListener(this);
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		lstSymbols.setEnabled(enabled);
		btnOption.setEnabled(enabled);
	}

	@Override
	protected void initReader() {
		WaitDialog.show(this, R.string.barcode_load_param);

		new Thread(new Runnable() {

			@Override
			public void run() {
				ParamValueList values = getReader().getBarcodeParam(new ParamName[] { ParamName.UPCA, ParamName.UPCE0,
						ParamName.UPCE1, ParamName.EAN8, ParamName.EAN13, ParamName.Code128, ParamName.Code39,
						ParamName.Code93, ParamName.Code11, ParamName.I2of5, ParamName.Codabar, ParamName.MSI,
						ParamName.X2of5, ParamName.MacroPDF417, ParamName.MicroPDF417, ParamName.Matrix,
						ParamName.MaxiCode, ParamName.QRCode, ParamName.AztecCode, ParamName.NEC2of5, ParamName.R2of5,
						ParamName.A2of5, ParamName.GS1128, ParamName.Telepen, ParamName.GS1DataBarOmnidirectional, ParamName.GS1DataBarLimited,
						ParamName.GS1DataBarExpanded, ParamName.TriopticCode, ParamName.CodablockA, ParamName.CodablockF,
						ParamName.GS1CompositeCodes, ParamName.TLC39, ParamName.ChinaPost, ParamName.KoreaPost,
						ParamName.HanXinCode });

				if (values.getLength() <= 0) {
					ASRLog.e(TAG, "ERROR. Failed to load barcode symbol state parameter");

					runOnUiThread(new Runnable() {

						@Override
						public void run() {
							WaitDialog.hide();

							AlertDialog.Builder alert = new AlertDialog.Builder(EnableSymbolActivity.this);
							alert.setTitle(R.string.system_alert);
							alert.setIcon(android.R.drawable.ic_dialog_alert);
							alert.setMessage(R.string.failed_load_barcode_option);
							alert.setCancelable(false);
							alert.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

								@Override
								public void onClick(DialogInterface dialog, int which) {
									finish();
								}
							});
							alert.show();
						}

					});
					return;
				}

				adpSymbols.clear();
				for (ParamValue value : values) {
					adpSymbols.addItem(value.getName(), value.getBoolean());
				}

				runOnUiThread(new Runnable() {

					@Override
					public void run() {

						adpSymbols.notifyDataSetChanged();
						// Enable action widget
						enableActionWidgets(true);

						WaitDialog.hide();
					}

				});
			}

		}).start();
	}

}
