package jp.co.asterisk.asreader.l251g.support.view.sub.barcode;

import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

import com.asreader.lib.barcode.spc.ParamValue;
import com.asreader.lib.barcode.spc.type.ParamName;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.exception.AsReaderException;

import java.nio.charset.Charset;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;

import jp.co.asterisk.asreader.l251g.R;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

public class GeneralOptionActivity extends RfidActivity implements OnClickListener {

	private static final String TAG = GeneralOptionActivity.class.getSimpleName();
	private static final String KEY_BARCODE_CHARSET = "barcode_charset";
	
	private TextView txtVideoReverse;
	private TextView txtCharset;
	private Button btnSave;
	private Button btnCancel;
	
	private int mVideoReverse;
	private String[] mVideoReverseValue = new String[]{"Video Reverse Off", "Video Reverse Only", "Video Reverse and Standard Bar Codes"};
	private Charset mCharset;
	
	private Handler mEvent;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_barcode_option_general);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayHomeAsUpEnabled(true);
		}

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		this.createReader();

		// Initialize Widget
		this.initWidgets();

		// Initialize Reader
		this.initReader();

		// Enable action widget
		this.enableActionWidgets(true);
		
		mEvent = new Handler();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		this.destroyReader();

		super.onDestroy();
	}

	
	@Override
	protected void onStart() {
		super.onStart();
		
		WaitDialog.show(this, R.string.barcode_load_param);
		new Thread(mLoadOption).start();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	protected void initWidgets() {
		
		txtVideoReverse = (TextView)findViewById(R.id.video_reverse);
		txtVideoReverse.setOnClickListener(this);
		txtCharset = (TextView)findViewById(R.id.barcode_charset);
		txtCharset.setOnClickListener(this);
		btnSave = (Button)findViewById(R.id.save);
		btnSave.setOnClickListener(this);
		btnCancel = (Button)findViewById(R.id.option_cancel);
		btnCancel.setOnClickListener(this);
		mCharset = Charset.defaultCharset();
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		txtVideoReverse.setEnabled(enabled);
		txtCharset.setEnabled(enabled);
		btnSave.setEnabled(enabled);
		btnCancel.setEnabled(enabled);
	}

	@Override
	protected void initReader() {

	}

	@Override
	public void onClick(View v) {
		switch(v.getId()) {
		case R.id.video_reverse:
			showVideoReverseDialog();
			break;
		case R.id.barcode_charset:
			showCharsetDialog();
			break;
		case R.id.save:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.save_option);
			new Thread(mSaveOption).start();
			break;
		case R.id.option_cancel:
			finish();
			break;
		}
		
	}
	
	private Runnable mSaveOption = new Runnable() {

		@Override
		public void run() {
			getReader().setBarcodeParam(new ParamValue(ParamName.VideoReverse, mVideoReverse));
			try {
				getReader().setCharset(mCharset);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, "ERROR. getCharset - %s", e.getResultCode());
			}
			
			SharedPreferences prefs = getSharedPreferences("rfid.reader.demo", MODE_PRIVATE);
			SharedPreferences.Editor editor = prefs.edit();
			editor.putString(KEY_BARCODE_CHARSET, mCharset.name());
			editor.apply();
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					finish();
				}

			});
		}
		
	};
	
	private Runnable mLoadOption = new Runnable() {

		@Override
		public void run() {
			
			mVideoReverse = getReader().getBarcodeParam(ParamName.VideoReverse).getValue();
			
			try {
				mCharset = getReader().getCharset();
			} catch(AsReaderException e) {
				ASRLog.e(TAG, "ERROR. getCharset - %s", e.getResultCode());
				mCharset = Charset.defaultCharset();
			}
			
			SharedPreferences prefs = getSharedPreferences("rfid.reader.demo", MODE_PRIVATE);
			String charset = prefs.getString(KEY_BARCODE_CHARSET, mCharset.name());
			try {
				mCharset = Charset.forName(charset);
				getReader().setCharset(mCharset);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, "ERROR. setCharset - %s", e.getResultCode());
			}
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					txtVideoReverse.setText(mVideoReverseValue[mVideoReverse]);
					txtCharset.setText(mCharset.name());
					enableActionWidgets(true);
					WaitDialog.hide();
				}
				
			});
		}
	
	};
	
	@SuppressWarnings("rawtypes")
	private void showCharsetDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		
		SortedMap<String, Charset> charSets = Charset.availableCharsets();
		Set s = charSets.entrySet();
		Iterator i = s.iterator();
		String[] charsetNames = new String[charSets.size()];
		int idx = 0;
		while(i.hasNext()) {
			Map.Entry m = (Map.Entry)i.next();
			Charset cs = (Charset)m.getValue();
			charsetNames[idx++] = cs.name();
		}
		adapter.addItem(charsetNames);
		
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.video_reverse);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mCharset = Charset.forName(adapter.getName(position));
				} catch (Exception e) {
				}
				
				txtCharset.setText(mCharset.name());
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mCharset.name());
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}

	private void showVideoReverseDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mVideoReverseValue.length; i++) {
			adapter.addItem(i, mVideoReverseValue[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.video_reverse);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mVideoReverse = adapter.getValue(position);
				} catch (Exception e) {
				}
				
				txtVideoReverse.setText(mVideoReverseValue[mVideoReverse]);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mVideoReverse);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}
}
