package jp.co.asterisk.asreader.l251g.support.view.sub.barcode.symbol;

import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.filter.InputFilterMinMax;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

import com.asreader.lib.barcode.spc.ParamValue;
import com.asreader.lib.barcode.spc.ParamValueList;
import com.asreader.lib.barcode.spc.type.ParamName;
import com.asreader.lib.diagnostics.ASRLog;

import java.util.Locale;

import jp.co.asterisk.asreader.l251g.R;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.text.InputFilter;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

public class Code11SymbolActivity extends RfidActivity implements OnClickListener {

	private static final String TAG = Code11SymbolActivity.class.getSimpleName();
	
	private ParamName mSymbolName;
	private TextView lblCheckDigit;
	private TextView txtCheckDigitsRequired;
	private TextView txtMinLen;
	private TextView txtMaxLen;
	private Button btnSave;
	private Button btnDefault;
	private Handler mEvent;
	
	private ParamName paramMinLength;
	private ParamName paramMaxLength;
	private ParamName paramCheckDigit;
	private ParamName paramDefault;
	
	private int mCheckDigit;
	private int mMinLen;
	private int mMaxLen;
	
	private int LENGTH_MIN = 0;
	private int LENGTH_MAX = 90;
	
	private String[] mCheckDigits;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_barcode_option_symbol_code11);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayHomeAsUpEnabled(true);
		}
		
		this.mSymbolName = (ParamName) getIntent().getSerializableExtra("Symbol_Name");

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		createReader();

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initReader();

		// Enable action widget
		enableActionWidgets(true);
		
		mEvent = new Handler();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		destroyReader();

		super.onDestroy();
	}
	
	

	@Override
	protected void onStart() {
		super.onStart();
		
		WaitDialog.show(this, R.string.barcode_load_param);
		new Thread(mLoadOption).start();
	}

	@Override
	protected void onStop() {
		super.onStop();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	protected void initWidgets() {
		
		txtCheckDigitsRequired = (TextView) findViewById(R.id.code11_check_digits_required);
		txtCheckDigitsRequired.setOnClickListener(this);
		txtMinLen = (TextView) findViewById(R.id.code11_length_min);
		txtMinLen.setOnClickListener(this);
		txtMaxLen = (TextView) findViewById(R.id.code11_length_max);
		txtMaxLen.setOnClickListener(this);
		btnSave = (Button) findViewById(R.id.save);
		btnSave.setOnClickListener(this);
		btnDefault = (Button) findViewById(R.id.option_default);
		btnDefault.setOnClickListener(this);
		
		if(this.mSymbolName == ParamName.Code11) {
			mCheckDigits = new String[] {"1 Check Digit", "2 Check Digits"};
			paramCheckDigit = ParamName.Code11CheckDigit;
			paramMinLength = ParamName.Code11LengthMin;
			paramMaxLength = ParamName.Code11LengthMax;
			paramDefault = ParamName.Code11Default;
		} else {
			this.setTitle(R.string.barcode_symbol_korean2of5_name);
			lblCheckDigit = (TextView) findViewById(R.id.label_code11_check_digits_required);
			lblCheckDigit.setText(R.string.korea_post_check_digit);
			mCheckDigits = new String[] {"Don't Transmit Check Digit", "Transmit Check Digit"};
			paramCheckDigit = ParamName.KoreaPostCheckDigit;
			paramMinLength = ParamName.KoreaPostLengthMin;
			paramMaxLength = ParamName.KoreaPostLengthMax;
			paramDefault = ParamName.KoreaPostDefault;
		}
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		btnSave.setEnabled(enabled);
		btnDefault.setEnabled(enabled);
	}

	@Override
	protected void initReader() {

	}
	
	private void displayValues() {
		txtCheckDigitsRequired.setText(mCheckDigits[mCheckDigit]);
		txtMinLen.setText(String.format(Locale.US, "%d", mMinLen));
		txtMaxLen.setText(String.format(Locale.US, "%d", mMaxLen));
	}
	
	private Runnable mLoadOption = new Runnable() {

		@Override
		public void run() {
			
			ParamValueList values = getReader().getBarcodeParam(new ParamName[] { paramCheckDigit, paramMinLength, paramMaxLength});
			
			mCheckDigit = values.getValue(paramCheckDigit);
			mMinLen = values.getValue(paramMinLength);
			mMaxLen = values.getValue(paramMaxLength);
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					displayValues();
					enableActionWidgets(true);
					WaitDialog.hide();
				}
				
			});
		}
		
	};
	
	private Runnable mDefaultOption = new Runnable() {

		@Override
		public void run() {

			getReader().setBarcodeParam(new ParamValue(paramDefault));
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					finish();
				}

			});
		}
		
	};
	
	private Runnable mSaveOption = new Runnable() {

		@Override
		public void run() {
			
			ParamValueList values = new ParamValueList();
			
			values.add(paramCheckDigit, mCheckDigit);
			values.add(paramMinLength, mMinLen);
			values.add(paramMaxLength, mMaxLen);
			
			getReader().setBarcodeParam(values);
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					finish();
				}

			});
		}
		
	};
	
	private void showCheckDigitsRequiredDialog() {
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for(int i=0; i<mCheckDigits.length; i++) {
			adapter.addItem(i, mCheckDigits[i]);
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		if(mSymbolName == ParamName.Code11)
			builder.setTitle(R.string.code11_check_digits_required);
		else
			builder.setTitle(R.string.korea_post_check_digit);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mCheckDigit = adapter.getValue(position);
				} catch (Exception e) {
				}
				displayValues();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mCheckDigit);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showStartStopCharDialog().onShow()");
			}
		});
		dialog.show();
	}
	
	private void showMinLengthDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(LENGTH_MIN, LENGTH_MAX - 10) };
		value.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.barcode_length_min);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mMinLen = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {

				}
				displayValues();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
					ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$PositiveButton.onClick()");
				}
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
					ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$NegativeButton.onClick()");
				}
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
					ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onCancel()");
				}
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mMinLen));
				value.selectAll();
				value.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
					ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog()");
				}
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onShow()");
	}
	
	private void showMaxLengthDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(LENGTH_MIN, LENGTH_MAX) };
		value.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.barcode_length_max);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mMaxLen = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {

				}
				displayValues();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
					ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$PositiveButton.onClick()");
				}
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
					ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$NegativeButton.onClick()");
				}
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
					ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onCancel()");
				}
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mMaxLen));
				value.selectAll();
				value.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) {
					imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
					ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog()");
				}
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onShow()");
	}

	@Override
	public void onClick(View v) {
		switch(v.getId()) {
		case R.id.code11_check_digits_required:
			showCheckDigitsRequiredDialog();
			break;
		case R.id.code11_length_min:
			showMinLengthDialog();
			break;
		case R.id.code11_length_max:
			showMaxLengthDialog();
			break;
		case R.id.save:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.save_option);
			new Thread(mSaveOption).start();
			break;
		case R.id.option_default:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.load_default_option);
			new Thread(mDefaultOption).start();
			break;
		}
		
	}

}
