package jp.co.asterisk.asreader.l251g.support.view.sub.barcode.symbol;

import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.filter.InputFilterMinMax;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

import com.asreader.lib.barcode.spc.ParamValue;
import com.asreader.lib.barcode.spc.ParamValueList;
import com.asreader.lib.barcode.spc.type.ParamName;
import com.asreader.lib.diagnostics.ASRLog;

import java.util.Locale;

import jp.co.asterisk.asreader.l251g.R;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.text.InputFilter;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

public class LengthSymbolActivity extends RfidActivity  implements OnClickListener{

	private static final String TAG = LengthSymbolActivity.class.getSimpleName();
	
	private ParamName mSymbolName;
	
	private TextView txtMinLen;
	private TextView txtMaxLen;
	private Button btnSave;
	private Button btnDefault;
	private Handler mEvent;
	
	private int mMinLen;
	private int mMaxLen;
	
	private ParamName paramMinLength;
	private ParamName paramMaxLength;
	private ParamName paramDefault;
	private int LENGTH_MIN = 0;
	private int LENGTH_MAX;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_barcode_option_symbol_length);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayHomeAsUpEnabled(true);
		}
		
		this.mSymbolName = (ParamName) getIntent().getSerializableExtra("Symbol_Name");

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		this.createReader();

		// Initialize Widget
		this.initWidgets();

		// Initialize Reader
		this.initReader();

		// Enable action widget
		this.enableActionWidgets(true);
		
		mEvent = new Handler();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		this.destroyReader();

		super.onDestroy();
	}
	
	

	@Override
	protected void onStart() {
		super.onStart();
		
		WaitDialog.show(this, R.string.barcode_load_param);
		new Thread(mLoadOption).start();
	}

	@Override
	protected void onStop() {
		super.onStop();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	protected void initWidgets() {
		txtMinLen = (TextView) findViewById(R.id.pdf417_length_min);
		txtMinLen.setOnClickListener(this);
		txtMaxLen = (TextView) findViewById(R.id.pdf417_length_max);
		txtMaxLen.setOnClickListener(this);
		btnSave = (Button) findViewById(R.id.save);
		btnSave.setOnClickListener(this);
		btnDefault = (Button) findViewById(R.id.option_default);
		btnDefault.setOnClickListener(this);
		
		if(this.mSymbolName == ParamName.R2of5) {
			this.setTitle(R.string.barcode_symbol_r2of5_name);
			paramMinLength = ParamName.R2of5LengthMin;
			paramMaxLength = ParamName.R2of5LengthMax;
			paramDefault = ParamName.R2of5Default;
			LENGTH_MAX = 48;
		} else if(this.mSymbolName == ParamName.A2of5) {
			this.setTitle(R.string.barcode_symbol_a2of5_name);
			paramMinLength = ParamName.A2of5LengthMin;
			paramMaxLength = ParamName.A2of5LengthMax;
			paramDefault = ParamName.A2of5Default;
			LENGTH_MAX = 48;
		} else if(this.mSymbolName == ParamName.CodablockA) {
			this.setTitle(R.string.barcode_symbol_codablocka_name);
			paramMinLength = ParamName.CodablockALengthMin;
			paramMaxLength = ParamName.CodablockALengthMax;
			paramDefault = ParamName.CodablockADefault;
			LENGTH_MAX = 600;
		} else if(this.mSymbolName == ParamName.CodablockF) {
			this.setTitle(R.string.barcode_symbol_codablockf_name);
			paramMinLength = ParamName.CodablockFLengthMin;
			paramMaxLength = ParamName.CodablockFLengthMax;
			paramDefault = ParamName.CodablockFDefault;
			LENGTH_MAX = 2048;
		} else if(this.mSymbolName == ParamName.MicroPDF417) {
			this.setTitle(R.string.barcode_symbol_micro_pdf417_name);
			paramMinLength = ParamName.MicroPDF417LengthMin;
			paramMaxLength = ParamName.MicroPDF417LengthMax;
			paramDefault = ParamName.MicroPDF417Default;
			LENGTH_MAX = 366;
		} else if(this.mSymbolName == ParamName.MaxiCode) {
			this.setTitle(R.string.barcode_symbol_maxicode_name);
			paramMinLength = ParamName.MaxiCodeLengthMin;
			paramMaxLength = ParamName.MaxiCodeLengthMax;
			paramDefault = ParamName.MaxiCodeDefault;
			LENGTH_MAX = 150;
		} else if(this.mSymbolName == ParamName.HanXinCode) {
			this.setTitle(R.string.barcode_symbol_hanxin_name);
			paramMinLength = ParamName.HanXinCodeLengthMin;
			paramMaxLength = ParamName.HanXinCodeLengthMax;
			paramDefault = ParamName.HanXinCodeDefault;
			LENGTH_MAX = 7833;
		} else if(this.mSymbolName == ParamName.ChinaPost) {
			this.setTitle(R.string.barcode_symbol_china_post_name);
			paramMinLength = ParamName.ChinaPostLengthMin;
			paramMaxLength = ParamName.ChinaPostLengthMax;
			paramDefault = ParamName.ChinaPostDefault;
			LENGTH_MAX = 80;
		} else if(this.mSymbolName == ParamName.GS1128) {
			this.setTitle(R.string.barcode_symbol_gs1128_name);
			paramMinLength = ParamName.GS1128LengthMin;
			paramMaxLength = ParamName.GS1128LengthMax;
			paramDefault = ParamName.GS1128Default;
			LENGTH_MAX = 80;
		} else if(this.mSymbolName == ParamName.X2of5) {
			this.setTitle(R.string.barcode_symbol_matrix2of5_name);
			paramMinLength = ParamName.X2of5LengthMin;
			paramMaxLength = ParamName.X2of5LengthMax;
			paramDefault = ParamName.X2of5Default;
			LENGTH_MAX = 80;
		} else if(this.mSymbolName == ParamName.GS1DataBarExpanded) {
			this.setTitle(R.string.barcode_symbol_rss_expanded_name);
			paramMinLength = ParamName.GS1DataBarExpandedLengthMin;
			paramMaxLength = ParamName.GS1DataBarExpandedLengthMax;
			paramDefault = ParamName.GS1DataBarExpandedDefault;
			LENGTH_MAX = 74;
		} else if(this.mSymbolName == ParamName.PDF417) {
			this.setTitle(R.string.barcode_symbol_pdf417_name);
			paramMinLength = ParamName.PDF417LengthMin;
			paramMaxLength = ParamName.PDF417LengthMax;
			paramDefault = ParamName.PDF417Default;
			LENGTH_MAX = 2750;
		}
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		btnSave.setEnabled(enabled);
		btnDefault.setEnabled(enabled);
	}
	
	private void displayValues() {
		txtMinLen.setText(String.format(Locale.US, "%d", mMinLen));
		txtMaxLen.setText(String.format(Locale.US, "%d", mMaxLen));
	}
	
	private Runnable mLoadOption = new Runnable() {

		@Override
		public void run() {
			
		ParamValueList values = getReader().getBarcodeParam(new ParamName[] { paramMinLength, paramMaxLength});

		mMinLen = values.getValue(paramMinLength);
		mMaxLen = values.getValue(paramMaxLength);

		mEvent.post(new Runnable() {

			@Override
			public void run() {
				displayValues();
				enableActionWidgets(true);
				WaitDialog.hide();
			}

		});
		}
		
	};
	
	private Runnable mDefaultOption = new Runnable() {

		@Override
		public void run() {

		getReader().setBarcodeParam(new ParamValue(paramDefault));

		mEvent.post(new Runnable() {

			@Override
			public void run() {
				WaitDialog.hide();
				finish();
			}

		});
		}
		
	};
	
	private Runnable mSaveOption = new Runnable() {

		@Override
		public void run() {
			
			ParamValueList values = new ParamValueList();
			
			values.add(paramMinLength, mMinLen);
			values.add(paramMaxLength, mMaxLen);
			
			getReader().setBarcodeParam(values);
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					finish();
				}

			});
		}
		
	};
	
	private void showMinLengthDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(LENGTH_MIN, LENGTH_MAX) };
		value.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.barcode_length_min);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mMinLen = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {

				}
				displayValues();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mMinLen));
				value.selectAll();
				value.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onShow()");
	}
	
	private void showMaxLengthDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_number, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(LENGTH_MIN, LENGTH_MAX) };
		value.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.barcode_length_max);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mMaxLen = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {

				}
				displayValues();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mMaxLen));
				value.selectAll();
				value.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null) imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaqxQDialog().onShow()");
	}

	@Override
	protected void initReader() {

	}

	@Override
	public void onClick(View v) {
		switch(v.getId()) {
		case R.id.pdf417_length_min:
			showMinLengthDialog();
			break;
		case R.id.pdf417_length_max:
			showMaxLengthDialog();
			break;
		case R.id.save:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.save_option);
			new Thread(mSaveOption).start();
			break;
		case R.id.option_default:
			enableActionWidgets(false);
			WaitDialog.show(this, R.string.load_default_option);
			new Thread(mDefaultOption).start();
			break;
		}
	}

}
