package jp.co.asterisk.asreader.l251g.view;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.PowerManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.TextView;

import jp.co.asterisk.asreader.l251g.MainActivity;
import jp.co.asterisk.asreader.l251g.R;
import com.asreader.lib.AsReader;
import com.asreader.lib.barcode.BarcodeType;
import com.asreader.lib.device.type.ConnectionState;
import com.asreader.lib.device.type.KeyState;
import com.asreader.lib.device.type.KeyType;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.R2000.R2000Model;
import com.asreader.lib.rfid.exception.AsReaderException;
import com.asreader.lib.rfid.type.ActionState;

import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Locale;

import jp.co.asterisk.asreader.l251g.support.adapter.BarcodeListAdapter;
import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.data.MySQLiteOpenHelper;
import jp.co.asterisk.asreader.l251g.support.data.SaveAsCSV;
import jp.co.asterisk.asreader.l251g.support.data.SaveAsExcel;
import jp.co.asterisk.asreader.l251g.support.util.ResUtil;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidClearActivity;

import static jp.co.asterisk.asreader.l251g.R.string.rescan_time;

public class BarcodeDemoActivity extends RfidClearActivity implements OnClickListener {

	private static final String TAG = BarcodeDemoActivity.class.getSimpleName();

	private static final int VIEW_BARCODE_OPTION = 1000;

	private TextView txtCount;
	private TextView txtRescanTime;
	private Button btnStartAction;
	private Button btnStopAction;
	private Button btnScanHW;
	private Button btnRescan;
	private RadioButton rbtnAscii;
	private RadioButton rbtnUTF8;
	private RadioButton rbtnShiftJIS;
	private MenuItem mnuSaveFile;
	private MenuItem mnuSaveExcel;

	private BarcodeListAdapter adpBarcode;
	private boolean isScanning;
	private boolean isScanningHw;

	// Rescan
	private boolean isReScanning = true;
	private String[] mArrayRescanTime = new String[]{"Not Used","100 ms","200 ms","300 ms","400 ms", "500 ms", "600 ms", "700 ms", "800 ms",
			"900 ms", "1000 ms", "1100 ms", "1200 ms", "1300 ms", "1500 ms", "2000 ms", "2500 ms", "3000 ms", "3500 ms", "5000 ms"};

	// DB
	protected MySQLiteOpenHelper dbHelper;

	// Encoding
	private Charset mCharset_ASCii = Charset.forName("ASCII");
	private Charset mCharset_UTF8 = Charset.forName("UTF-8");
	private Charset mCharset_ShiftJIS = Charset.forName("Shift_JIS");

	private static final Object objLock = new Object();

	PowerManager powerManager;
	PowerManager.WakeLock wakeLock;


	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_barcode_demo);

		// Android OS screen power saving mode control.
		powerManager = (PowerManager) getSystemService(POWER_SERVICE);
		wakeLock = powerManager.newWakeLock(PowerManager.FULL_WAKE_LOCK, "BarcodeScan:WakeLock");

		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
			actionBar.setCustomView(R.layout.activity_title);
			actionBar.setDisplayHomeAsUpEnabled(true);
			actionBar.setDisplayShowHomeEnabled(true);
		}
		
		TextView titleText = (TextView)findViewById(R.id.title_text);
		titleText.setText(R.string.barcode_demo_name);
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			titleText.setTextColor(getColor(getApplicationContext(), R.color.black));

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		createReader();

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initReader();

		txtRescanTime.setText((MainActivity.mUserConfigs.getDecodeRescanTime()==0)?"Not Used":MainActivity.mUserConfigs.getDecodeRescanTime()+" ms");
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		try {
			getReader().setBarcodeMode(false);
		} catch (AsReaderException e) {
			e.printStackTrace();
		}

		// Destroy Reader
		destroyReader();

		unregisterReceiver(screenOnReceiver);
		super.onDestroy();
	}

	@Override
	protected void onStart() {
		super.onStart();
	}

	@Override
	protected void onStop() {
		if(getReader().getAction() == ActionState.StartDecode)
			getReader().stopDecode();

		dbHelper.deleteBarcode();

		// DB insert
		for (int i =0; i<adpBarcode.getCount(); i++){
//			Log.d(TAG,"# DB stop : set DB Barcode List item : " + adpBarcode.getBarcodeItemForDB(i));

			//Barcode, count, type, codeid, time
			String[] temp = adpBarcode.getBarcodeItemForDB(i);
			dbHelper.insertBarcode(temp[0],temp[1],temp[2],temp[3], temp[4]);
		}

		super.onStop();
	}

	@Override
	protected void onResume() {
		super.onResume();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onResume()");
		R2000Model.getInstance().isTriggerEnable = true;
	}

	@Override
	protected void onPause() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onPause()");
		R2000Model.getInstance().isTriggerEnable = false;
		super.onPause();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

		if (requestCode == VIEW_BARCODE_OPTION) {
			enableActionWidgets(true);
		}
		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public void onClick(View v) {
		super.onClick(v);

		int id = v.getId();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(id));

		switch (id) {
		case R.id.start_action:
			if (!getIsScanning())
				scanEnabled(true);
			break;

		case R.id.stop_action:
			if (getIsScanning())
				scanEnabled(false);
			break;

		case R.id.btn_rescan:
			showRescanTimeDialog();
			break;

		case R.id.scanHW:
			// H/W Barcode Settings
			if (getIsHWScanning()) {
				getReader().startScanHW();
				scanHWEnabled(true);
				setIsHWScanning(false);
			}else {
				getReader().stopScanHW();
				scanHWEnabled(false);
				setIsHWScanning(true);
			}
			break;

		case R.id.ascii:
			encodingType(mCharset_ASCii);
			break;
		case R.id.utf8:
			encodingType(mCharset_UTF8);
			break;
		case R.id.shiftjis:
			encodingType(mCharset_ShiftJIS);
			break;
		}
	}

	private void encodingType(Charset charset){
		try {
			getReader().setCharset(charset);
		} catch (AsReaderException e) {
			e.printStackTrace();
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateOptionsMenu()");
		getMenuInflater().inflate(R.menu.save_file, menu);
		return true;
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		mnuSaveFile = menu.findItem(R.id.save_csv);
		mnuSaveExcel = menu.findItem(R.id.save_excel);
		mnuSaveFile.setEnabled(getReader().getAction() == ActionState.Stop);
		mnuSaveExcel.setEnabled(getReader().getAction() == ActionState.Stop);
		return super.onPrepareOptionsMenu(menu);
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		int id = item.getItemId();

		switch (id) {
		case R.id.save_csv:
			if(adpBarcode.getCount() < 1)
				return true;
			
			SaveAsCSV task = new SaveAsCSV(BarcodeDemoActivity.this, SaveAsCSV.DATA_TYPE_BARCODE, adpBarcode, false);
			task.execute();
			return true;

			case R.id.save_excel:
				if(adpBarcode.getCount() < 1)
					return true;

				SaveAsExcel excel = new SaveAsExcel(BarcodeDemoActivity.this, SaveAsExcel.DATA_TYPE_BARCODE, adpBarcode, false);
				excel.write();
				return true;
		}
		return super.onOptionsItemSelected(item);
	}

	public boolean getIsScanning() {
		synchronized(objLock) {
			return isScanning;
		}
	}
	public void setIsScanning(boolean value) {
		synchronized(objLock) {
			isScanning = value;
		}
	}

	public boolean getIsHWScanning() {
		synchronized(objLock) {
			return isScanningHw;
		}
	}
	public void setIsHWScanning(boolean value) {
		synchronized(objLock) {
			isScanningHw = value;
		}
	}
	
	@Override
	public void onStateChanged(AsReader reader, ConnectionState state) {
		if (state == ConnectionState.Disconnected) {
			setResult(RESULT_DISCONNECTED);
			finish();
		}
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onStateChanged(%s)", state);
	}


	@Override
	public void onActionChanged(AsReader reader, ActionState action) {

		if (action == ActionState.StartDecode) {
			scanElemEnabled(true);

		} else if (action == ActionState.Stop) {
			int mRescanTime = MainActivity.mUserConfigs.getDecodeRescanTime();

			// Rescan Time > 0
			if (mRescanTime>0 && isReScanning){
				new Handler().postDelayed(new Runnable() {
					@Override
					public void run() {
						if (isReScanning) {
							getReader().startDecode();
						}
					}
				}, mRescanTime);
			}else {
				scanElemEnabled(false);
				scanHWEnabled(false);
			}

		}

		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onActionChanged(%s)", action);
	}

	@Override
	public void onReadBarcode(AsReader reader, BarcodeType type, String codeId, String barcode) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onReadBarcode (%s, %s, [%s], %d bytes)", type, codeId, barcode, barcode.length());
		adpBarcode.addItem(type, codeId, barcode, System.currentTimeMillis());
		txtCount.setText(String.format(Locale.US, "%d", adpBarcode.getCount()));
	}

	// Initialize Reader
	protected void initReader() {
		// encoding type reset.
		try {
			getReader().setCharset(Charset.forName("ASCII"));
			rbtnAscii.setChecked(true);
		} catch (AsReaderException e) {
			e.printStackTrace();
		}

		setIsScanning(false);
		setIsHWScanning(true);
		mnuSaveFile = null;

		try {
			getReader().setBarcodeMode(true);

			getReader().setBuzzer(getReader().getBuzzer());
			getReader().setVibratorState(getReader().getVibratorState());

		} catch (AsReaderException e) {
			e.printStackTrace();
		}
		// Blocked trigger event when off screen.
		R2000Model.getInstance().isTriggerEnable = true;
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initReader()");
	}

	protected void initWidgets() {
		super.initWidgets();
		// new database
		dbHelper = MySQLiteOpenHelper.getInstance(this);

		ListView lstBarcodes = (ListView) findViewById(R.id.barcodes);
		adpBarcode = new BarcodeListAdapter(this);

		// data from db
		ArrayList dbBarcode = dbHelper.selectBarcode();

		try {
			for (int i = 0; i < dbBarcode.size(); i++) {
				//Barcode, count, type, codeid, time
				String[] temp = dbBarcode.get(i).toString().split("<split>");
				BarcodeType type = super.getBarcodeType(temp[3]);

//			 type, count, codeId, barcode, time
				adpBarcode.addItem(type, Integer.parseInt(temp[1]), temp[3], temp[0], Long.parseLong(temp[4]));
			}
		}catch (Exception e){
			e.getStackTrace();
		}

		lstBarcodes.setAdapter(adpBarcode);

		txtCount = (TextView) findViewById(R.id.barcode_count);
		txtCount.setText(String.format(Locale.US, "%d", adpBarcode.getCount()));
		btnScanHW = (Button) findViewById(R.id.scanHW);
		btnScanHW.setOnClickListener(this);
		btnStartAction = (Button) findViewById(R.id.start_action);
		btnStartAction.setOnClickListener(this);
		btnStopAction = (Button) findViewById(R.id.stop_action);
		btnStopAction.setOnClickListener(this);
		btnRescan = (Button) findViewById(R.id.btn_rescan);
		btnRescan.setOnClickListener(this);
		rbtnAscii = (RadioButton) findViewById(R.id.ascii);
		rbtnAscii.setOnClickListener(this);
		rbtnUTF8 = (RadioButton) findViewById(R.id.utf8);
		rbtnUTF8.setOnClickListener(this);
		rbtnShiftJIS = (RadioButton) findViewById(R.id.shiftjis);
		rbtnShiftJIS.setOnClickListener(this);

		txtRescanTime = (TextView) findViewById(R.id.tv_rescan_time);

		// Enable action widget
		enableActionWidgets(getReader().isBarcodeEnabled());

		// Screen On/Off event
		useScreenOnOffMonitoring();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. initWidgets()");
	}


	@Override
	protected void clearWidgets() {
		dbHelper.deleteBarcode();
		adpBarcode.clear();
		txtCount.setText(String.format(Locale.US, "%d", adpBarcode.getCount()));
		enableActionWidgets(true);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. clearWidgets()");
	}

	// Enable/Disable Action Widgets
	protected void enableActionWidgets(boolean enabled) {
		btnStartAction.setEnabled(!getIsScanning() && enabled);
		btnStopAction.setEnabled(getIsHWScanning());
		btnScanHW.setEnabled(enabled);
		btnClear.setEnabled(true);
		btnRescan.setEnabled(!getIsScanning() && enabled);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}

	@Override
	protected String getRequestCode(int requestCode) {
		switch (requestCode) {
		case VIEW_BARCODE_OPTION:
			return "VIEW_BARCODE_OPTION";
		}
		return super.getRequestCode(requestCode);
	}

	@Override
	public void onExceptionOccurred(AsReader reader, int code) {
		ASRLog.e(TAG, "DEBUG. onExceptionOccurred : 0x%X", code);
	}


	@Override
	public void onKeyEvent(AsReader reader, KeyType key, KeyState state) {
		if(key == KeyType.Trigger) {
			if (state == KeyState.KeyDown) {
				isReScanning = true;
				scanEnabled(true);
			}else {
				isReScanning = false;
				scanEnabled(false);
			}
		}
	}

	private void scanHWEnabled(boolean enabled){
		if (enabled){
			btnScanHW.setText(R.string.action_scan_stop_hw);
//			setIsHWScanning(false);
			enableActionWidgets(false);
		}else {
			btnScanHW.setText(R.string.action_scan_hw);
			enableActionWidgets(true);
//			setIsHWScanning(true);
		}
	}

	private void scanEnabled(boolean enabled){
		if (enabled){
			getReader().startDecode();
			scanElemEnabled(true);

			if (wakeLock!=null)
				wakeLock.acquire();		// Preventing screen sleep during scanning.
		}else {
			getReader().stopDecode();
			scanElemEnabled(false);

			if (wakeLock!=null&&wakeLock.isHeld())
				wakeLock.release();		// After the scan is finished, cancels blocking the screen sleep mode.
		}
	}

	private void scanElemEnabled(boolean enabled){
		isReScanning = enabled;
		setIsScanning(enabled);
		enableActionWidgets(enabled);
	}


	private void showRescanTimeDialog() {

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		adapter.addItem(mArrayRescanTime);				// 임시 데이터
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(rescan_time);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position;
				try {
					position = value.getCheckedItemPosition();
					adapter.getValue(position);
				}catch (ArrayIndexOutOfBoundsException e){
					Log.d(TAG, e.getMessage());
					position = 0;
				}

				value.setItemChecked(position,true);
				txtRescanTime.setText(adapter.getName(position));
				String[] str_temp = adapter.getName(position).split(" ");
				int value;
				if (str_temp[0].contains("Not"))
					value = 0;
				else
					value = Integer.parseInt(str_temp[0]);

				MainActivity.mUserConfigs.setDecodeRescanTime(value);
				MainActivity.mUserConfigs.Save();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showRescanTimeDialog().$PositiveButton.onClick()");
			}
		});
		builder.setCancelable(true);
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(MainActivity.mUserConfigs.getDecodeRescanTime());
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showRescanTimeDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showRescanTimeDialog()");
	}


	/************************************************************
	 *  Screen On/Off Event
	 ************************************************************/
	private ScreenOnReceiver screenOnReceiver =null;
	class ScreenOnReceiver extends BroadcastReceiver {
		public void onReceive(Context context, Intent intent) {
			String action = intent.getAction();
			if (action.equals(Intent.ACTION_SCREEN_ON)) {
				R2000Model.getInstance().isTriggerEnable = true;
			} else if (action.equals(Intent.ACTION_SCREEN_OFF)) {
				R2000Model.getInstance().isTriggerEnable = false;
			}
		}
	}

	private void useScreenOnOffMonitoring()
	{
		if(screenOnReceiver == null)
		{
			screenOnReceiver = new ScreenOnReceiver();
			IntentFilter filter = new IntentFilter();
			filter.addAction("android.intent.action.SCREEN_ON");
			filter.addAction("android.intent.action.SCREEN_OFF");
			registerReceiver(screenOnReceiver, filter);
		}
	}
}

