package jp.co.asterisk.asreader.l251g.view;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.PowerManager;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import jp.co.asterisk.asreader.l251g.MainActivity;
import jp.co.asterisk.asreader.l251g.R;
import com.asreader.lib.AsReader;
import com.asreader.lib.device.type.ConnectionState;
import com.asreader.lib.device.type.ExceptionCode;
import com.asreader.lib.device.type.KeyState;
import com.asreader.lib.device.type.KeyType;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.R2000.R2000Model;
import com.asreader.lib.rfid.exception.AsReaderException;
import com.asreader.lib.rfid.params.PowerRange;
import com.asreader.lib.rfid.type.ActionState;
import com.asreader.lib.rfid.type.QuerySession;
import com.asreader.lib.rfid.type.QueryTarget;
import com.asreader.lib.rfid.type.ResultCode;

import java.util.Locale;

import jp.co.asterisk.asreader.l251g.support.adapter.PowerRangeAdapter;
import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.data.SaveAsCSV;
import jp.co.asterisk.asreader.l251g.support.data.SaveAsExcel;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.util.ResUtil;
import jp.co.asterisk.asreader.l251g.support.view.base.AccessActivity;
import jp.co.asterisk.asreader.l251g.support.view.base.BaseInventoryActivity;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidActivity;

public class InventoryActivity extends BaseInventoryActivity {

	private static final String TAG = InventoryActivity.class.getSimpleName();

//	private TextView txtTotalCount;
	private TextView txtSpeed;
	private LinearLayout layoutOption;
	private TextView txtPowerGain;
	private TextView txtDisplayPC;
	private TextView txtContinuousMode;
	private TextView txtReportRssi;
	private TextView txtOperationTime;
	private TextView txtQuerySession;
	private TextView txtQueryTarget;
	// rfm sensor
	private Button btnRfmSensor;
	
	private PowerRange mPowerRange;
	private int mCurrentPowerGain;
	private boolean mDisplayPC;
	private boolean mContinuousMode;
	private boolean mReportRssi;
	private int mOperationTime;
	private QuerySession mQuerySession;
	private QueryTarget mQueryTarget;

//	private long mTotalTagCount;
//	private long mLastTime;
//	private double mTagSpeed;

	private static final String KEY_LAST_CONTINUOUS_MODE = "last_continuous_mode";

	private int mSpeedTagCount;
	private long mStartCalcTime = 0;
	private boolean isTriggerEnable = false;

	private int mRestartTime;
	private boolean mIsRunRestart;
	private Handler mHandler;

	PowerManager powerManager;
	PowerManager.WakeLock wakeLock;


	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_inventory);

		// Android OS screen power saving mode control.
		powerManager = (PowerManager) getSystemService(POWER_SERVICE);
		wakeLock = powerManager.newWakeLock(PowerManager.FULL_WAKE_LOCK, "Inventory:WakeLock");

		isTriggerEnable = false;

		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
			actionBar.setCustomView(R.layout.activity_title);
			actionBar.setDisplayHomeAsUpEnabled(true);
			actionBar.setDisplayShowHomeEnabled(true);
		}
		TextView titleText = (TextView)findViewById(R.id.title_text);
		titleText.setText(R.string.inventory_name);
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			titleText.setTextColor(getColor(getApplicationContext(), R.color.black));
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		mHandler = new Handler();
		
		// Create Reader
		createReader();

		// Initialize Mask
		initMask();

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initReader();

//		mLastTime = 0;
//		mTagSpeed = 0.0;
		mRestartTime = 0;
		mIsRunRestart = false;
		
		// Enable action widget
		enableActionWidgets(true);

		mSpeedTagCount = 0;
		mStartCalcTime = 0;
		mCurrentPowerGain = 0;
		mDisplayPC = true;
		mContinuousMode = false;
		mReportRssi = false;
		mOperationTime = 0;
		mQuerySession = QuerySession.S0;
		mQueryTarget = QueryTarget.AB;

		if(getReader().getState() == ConnectionState.Connected) {
			// Load Option
			WaitDialog.show(this, R.string.load_option);
			new Thread(mLoadOption).start();
		}

		// Blocked trigger event when off screen.
//		if (isTriggerEnable)
//			R2000Model.getInstance().isTriggerEnable = true;

		// sensor test
		btnRfmSensor = (Button) findViewById(R.id.rfm);
		btnRfmSensor.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				R2000Model.getInstance().isRfmStopRollback = false;
				getReader().readRFMSensorTag();
				adpTags.setReportTID(true); // enable TID Text
			}
		});

		useScreenOnOffMonitoring();

	}

	@Override
	protected void onResume() {
		// Blocked trigger event when off screen.
		if (isTriggerEnable)
			R2000Model.getInstance().isTriggerEnable = true;

		super.onResume();
	}

	@Override
	protected void onPause() {
		R2000Model.getInstance().isTriggerEnable = false;
		super.onPause();
	}

	@Override
	protected void onDestroy() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		destroyReader();

		adpTags.shutDown();

		unregisterReceiver(screenOnReceiver);

		super.onDestroy();
	}
	
	@Override
	public void onBackPressed() {
		if(layoutOption.getVisibility() == View.VISIBLE) {
			layoutOption.setVisibility(View.GONE);
			return;
		}
		super.onBackPressed();
	}

	@Override
	protected void onStart() {
		super.onStart();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onStart()");
	}

	@Override
	protected void onStop() {
		if(getReader().getAction() == ActionState.Inventory)
			getReader().stop();
		super.onStop();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreateOptionsMenu()");
		getMenuInflater().inflate(R.menu.save_file, menu);
		return true;
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		MenuItem mnuSaveFile;
		MenuItem mnuSaveExcel;

		mnuSaveFile = menu.findItem(R.id.save_csv);
		mnuSaveFile.setEnabled(getReader().getAction() == ActionState.Stop);
		mnuSaveExcel = menu.findItem(R.id.save_excel);
		mnuSaveExcel.setEnabled(getReader().getAction() == ActionState.Stop);
		
		return super.onPrepareOptionsMenu(menu);
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		int id = item.getItemId();

		switch (id) {
			case R.id.save_csv:
				if(adpTags.getCount() < 1)
					return true;

				SaveAsCSV task = new SaveAsCSV(InventoryActivity.this, SaveAsCSV.DATA_TYPE_RFID, adpTags, mDisplayPC);
				task.execute();
				return true;

			case R.id.save_excel:
				if(adpTags.getCount() < 1)
					return true;

				SaveAsExcel excel = new SaveAsExcel(InventoryActivity.this, SaveAsExcel.DATA_TYPE_RFID, adpTags, mDisplayPC);
				excel.write();
				return true;
		}

		return super.onOptionsItemSelected(item);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		
		switch (requestCode) {
		case AccessActivity.VIEW_READ_MEMORY:
		case AccessActivity.VIEW_WRITE_MEMORY:
		case AccessActivity.VIEW_TAG_ACCESS:
		case RfidActivity.VIEW_MASK:
			if(getReader().getState() == ConnectionState.Connected
			&& R2000Model.getInstance().isTriggerEnable) {
				// Load Option
				WaitDialog.show(this, R.string.load_option);
				new Thread(mLoadOption).start();
			}
			break;
		}
	}

	private void displayPowerGain() {
		txtPowerGain.setText(String.format(Locale.US, "%.1f dBm", (double) mCurrentPowerGain / 10.0));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayPowerGain(%d)", mCurrentPowerGain);
	}
	
	private void displayDisplayPC() {
		txtDisplayPC.setText(mDisplayPC ? "On" : "Off");
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayDisplayPC(%s)", mDisplayPC);
	}

	private void displayContinuousMode() {
		txtContinuousMode.setText(mContinuousMode ? "On" : "Off");
	}
	
	private void displayReportRSSI() {
		txtReportRssi.setText(mReportRssi ? "On" : "Off");
	}
	
	private void displayOperationTime() {
		txtOperationTime.setText(String.format(Locale.US, "%d ms", mOperationTime));
	}
	
	private void displayQuerySession() {
		txtQuerySession.setText(mQuerySession.toString());
	}
	
	private void displayQueryTarget() {
		txtQueryTarget.setText(mQueryTarget.toString());
	}

	private Runnable mLoadOption = new Runnable() {

		@Override
		public void run() {
			
			try {
				mPowerRange = getReader().getPowerGainRange();
				mCurrentPowerGain = getReader().getPowerGain();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. initReader() - Failed to get output power (power gain)");
			}
			try {
				mContinuousMode = getReader().getContinuousMode();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. initReader() - Failed to get continuous mode");
			}
			try {
				mReportRssi = getReader().getReportRSSI();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. initReader() - Failed to get report rssi");
			}
			try {
				mOperationTime = getReader().getOperationTime();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. initReader() - Failed to get operation time");
			}
			try {
				mQuerySession = getReader().getQuerySession();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. initReader() - Failed to get query session");
			}
			try {
				mQueryTarget = getReader().getQueryTarget();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. initReader() - Failed to get query target");
			}

			mDisplayPC = MainActivity.mUserConfigs.getDisplayPc();

			MainActivity.mUserConfigs.setMaskDataUpdate(getReader(), mMaskType);

			mHandler.post(new Runnable() {

				@Override
				public void run() {

					setDisplayPC(mDisplayPC);
					displayPowerGain();
					displayDisplayPC();
					displayContinuousMode();
					displayReportRSSI();
					displayOperationTime();
					displayQuerySession();
					displayQueryTarget();
					WaitDialog.hide();
					enableActionWidgets(true);
					isTriggerEnable = true;
					R2000Model.getInstance().isTriggerEnable = true;
				}

			});
		}
		
	};

	private void showPowerGainDialog() {

		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView power = (ListView) root.findViewById(R.id.list);
		final PowerRangeAdapter adapter = new PowerRangeAdapter(this, mPowerRange,
				android.R.layout.simple_list_item_single_choice);
		power.setAdapter(adapter);
		power.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.power_gain);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = power.getCheckedItemPosition();
				int power = adapter.getItemValue(position);
				
				try {
					getReader().setPowerGain(power);
					MainActivity.mUserConfigs.setPowerGain(power);
					MainActivity.mUserConfigs.Save();
					mCurrentPowerGain = power;
				} catch (AsReaderException e) {
					ASRLog.e(TAG, e, "ERROR. $showPowerGainDialog() - Failed to set output power (power gain) (%d)", power);
				}
				
				displayPowerGain();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showPowerGinaDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showPowerGinaDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {
			
			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showPowerGinaDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getItemPosition(mCurrentPowerGain);
				power.setItemChecked(position, true);
				power.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showPowerGinaDialog().onShow()");
			}
		});
		dialog.show();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. showPowerGinaDialog()");
	}
	
	private void showDisplayPcDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		adapter.addItem(0, "Off");
		adapter.addItem(1, "On");
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.display_pc);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mDisplayPC = adapter.getValue(position) > 0;
					getReader().setReportRSSI(mDisplayPC);
					setDisplayPC(mDisplayPC);
					MainActivity.mUserConfigs.setDisplayPc(mDisplayPC);
					MainActivity.mUserConfigs.Save();
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR.  showDisplayPcDialog().$PositiveButton.onClick() - [%s]"
								, e.getMessage());
				}
				displayDisplayPC();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showDisplayPcDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showDisplayPcDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showDisplayPcDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mDisplayPC ? 1 : 0);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showDisplayPcDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showDisplayPcDialog()");
	}
	
	private void showContinuousModeDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		adapter.addItem(0, "Off");
		adapter.addItem(1, "On");
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.continuous_mode);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mContinuousMode = adapter.getValue(position) > 0;
					getReader().setContinuousMode(mContinuousMode);
					saveConinuousMode();
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR.  showContinuousModeDialog().$PositiveButton.onClick() - [%s]"
							, e.getMessage());
				}
				displayContinuousMode();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showContinuousModeDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showContinuousModeDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showContinuousModeDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mContinuousMode ? 1 : 0);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showContinuousModeDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showContinuousModeDialog()");
	}
	
	private void showReportRssiDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		adapter.addItem(0, "Off");
		adapter.addItem(1, "On");
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.report_rssi);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mReportRssi = adapter.getValue(position) > 0;
					getReader().setReportRSSI(mReportRssi);
					MainActivity.mUserConfigs.setReportRssi(mReportRssi);
					MainActivity.mUserConfigs.Save();
					setReportRssiMode(mReportRssi);
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR.  showReportRssiDialog().$PositiveButton.onClick() - [%s]"
							, e.getMessage());
				}
				displayReportRSSI();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showReportRssiDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showReportRssiDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showReportRssiDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mReportRssi ? 1 : 0);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showReportRssiDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showReportRssiDialog()");
	}
	
	private void showOperationTimeDialog() {

		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
		final EditText operationTime = (EditText) root.findViewById(R.id.value);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.operation_time);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				
				try {
					mOperationTime = Integer.parseInt(operationTime.getText().toString());
				} catch (Exception e) {
					mOperationTime = 0;
				}
				try {
					getReader().setOperationTime(mOperationTime);
					MainActivity.mUserConfigs.setOperationTime(mOperationTime);
					MainActivity.mUserConfigs.Save();
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR.  showOperationTimeDialog().$PositiveButton.onClick() - [%s]"
							, e.getMessage());
				}
				displayOperationTime();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(operationTime.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showOperationTimeDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(operationTime.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showOperationTimeDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(operationTime.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showOperationTimeDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				operationTime.setText(String.format(Locale.US, "%d", mOperationTime));
				operationTime.selectAll();
				operationTime.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.showSoftInput(operationTime, InputMethodManager.SHOW_FORCED);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showOperationTimeDialog().onShow()");
			}
		});
		dialog.show();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. showOperationTimeDialog()");
	}
	
	private void showQuerySessionDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for (QuerySession item : QuerySession.values()) {
			adapter.addItem(item.getValue(), item.toString());
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.session);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mQuerySession = QuerySession.valueOf(adapter.getValue(position));
					getReader().setQuerySession(mQuerySession);
					MainActivity.mUserConfigs.setSession(mQuerySession);
					MainActivity.mUserConfigs.Save();
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR.  showQuerySessionDialog().$PositiveButton.onClick() - [%s]"
							, e.getMessage());
				}
				displayQuerySession();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQuerySessionDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQuerySessionDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQuerySessionDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mQuerySession.getValue());
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQuerySessionDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showQuerySessionDialog()");
	}
	
	private void showQueryTargetDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for (QueryTarget item : QueryTarget.values()) {
			adapter.addItem(item.getValue(), item.toString());
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.target);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mQueryTarget = QueryTarget.valueOf(adapter.getValue(position));
					getReader().setQueryTarget(mQueryTarget);
					MainActivity.mUserConfigs.setTarget(mQueryTarget);
					MainActivity.mUserConfigs.Save();
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR.  showQueryTargetDialog().$PositiveButton.onClick() - [%s]"
							, e.getMessage());
				}
				displayQueryTarget();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQueryTargetDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQueryTargetDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQueryTargetDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mQueryTarget.getValue());
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQueryTargetDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showQueryTargetDialog()");
	}
	
	private void showErrorDialog(int code) {
		
		final ExceptionCode readerException = ExceptionCode.valueOf(code);
		
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.exception_occurred);
		builder.setMessage(String.format(Locale.US, "Value : 0x%X\n%s", readerException.getValue(), readerException.toString()));
		builder.setCancelable(false);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {
				
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showErrorDialog().$PositiveButton.onClick()");
			}
		});
				
		AlertDialog dialog = builder.create();
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showAlgorithmDialog()");
	}

	@Override
	public void onClick(View v) {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));
		switch (v.getId()) {
		case R.id.option:
			if(layoutOption.getVisibility() == View.VISIBLE) {
				layoutOption.setVisibility(View.GONE);
			} else {
				layoutOption.setVisibility(View.VISIBLE);
			}
			break;
		case R.id.inventory_power_gain:
			showPowerGainDialog();
			break;
		case R.id.inventory_display_pc:
			showDisplayPcDialog();
			break;
		case R.id.inventory_continuous_mode:
			showContinuousModeDialog();
			break;
		case R.id.inventory_report_rssi:
			showReportRssiDialog();
			break;
		case R.id.inventory_operation_time:
			showOperationTimeDialog();
			break;
		case R.id.inventory_query_session:
			showQuerySessionDialog();
			break;
		case R.id.inventory_query_target:
			showQueryTargetDialog();
			break;
		}
		
		super.onClick(v);
	}


	@Override
	public void onReadTag(AsReader reader, ActionState action, String tag, float rssi, float phase, float frequency) {
		super.onReadTag(reader, action, tag, rssi, phase, frequency);

		/*
		long time = System.currentTimeMillis();
		//double interval = 0;
		int interval = 0;

		mSpeedTagCount++;
		if (mLastTime == 0) {
			mTagSpeed = 0.0;
			mLastTime = time;
		} else {
			//interval = (double) (time - mLastTime) / 1000.0;
			interval = (int) ((time - mLastTime) / 1000.0);
			mTagSpeed = (double) mSpeedTagCount / interval;
		}

		if (interval > 0) {
			txtSpeed.setText(String.format(Locale.US, "%.2f tps", mTagSpeed));
		} else {
			txtSpeed.setText("0.00 tps");
		}
		*/


		// check count every 1 second (1000 millisecond)
		mSpeedTagCount++;
		if(calcTPS() >= 1000l){
			txtSpeed.setText(String.valueOf(mSpeedTagCount)+" tps");
			mSpeedTagCount = 0;
		}
	}

	private long calcTPS(){
		if(mStartCalcTime ==0){
			mStartCalcTime = System.currentTimeMillis();
		}
		long currentTime = System.currentTimeMillis();
		long passedTime = currentTime - mStartCalcTime;

		if(passedTime >=1000l){
			mStartCalcTime = 0;
		}
		return passedTime;
	}


	@Override
	public void onReadTagWithRFM(AsReader reader, ActionState action, String tag, float rssi, float phase, float frequency, String[] rfm_data) {
		// rfm_data : tid, temperatureCode, calibration, sensorCode
		super.onReadTagWithRFM(reader, action, tag, rssi, phase, frequency, rfm_data);

		/*
		long time = System.currentTimeMillis();	// 해당 콜백이 들어올 때마다 time을 업데이트 함
		double interval;

		mSpeedTagCount++;
		if (mLastTime == 0) {
			mTagSpeed = 0.0;
			mLastTime = time;
		} else {
			interval = (double) (time - mLastTime) / 1000.0;
			mTagSpeed = (double) mSpeedTagCount / interval;
		}

		txtSpeed.setText(String.format(Locale.US, "%.2f tps", mTagSpeed));*/

		mSpeedTagCount++;
		if(calcTPS() >= 1000l){
			txtSpeed.setText(String.valueOf(mSpeedTagCount)+" tps");
			mSpeedTagCount = 0;
		}
	}

	@Override
	public void onActionChanged(AsReader reader, ActionState action) {
		
		if (mIsRunRestart) {
			adpTags.notifyDataSetChanged();
			if (action != ActionState.Stop) {
				enableActionWidgets(true);
			}
		} else {
			super.onActionChanged(reader, action);
		}
		
		if(action == ActionState.Stop){
			mSpeedTagCount = 0;
			mStartCalcTime = 0;
//			mLastTime =0;
			
			if (mRestartTime > 0 && mIsRunRestart) {
				mHandler.postDelayed(mRestartHandler, mRestartTime);
			}
		}
	}

	@Override
	public void onExceptionOccurred(AsReader reader, int code) {
		ASRLog.e(TAG, "DEBUG. onExceptionOccurred : 0x%X", code);
		showErrorDialog(code);
	}

	@Override
	public void onKeyEvent(AsReader reader, KeyType key, KeyState state) {
		if(key == KeyType.Trigger) {
			if(state == KeyState.KeyDown)
				startInventory();
			else
				stopInventory();
		}
	}

	// Initialize Reader
	@Override
	protected void initReader() {
		super.initReader();
		
		try {
			adpTags.setReportRSSI(getReader().getReportRSSI());
			getDB();
		} catch (AsReaderException e) {
			ASRLog.e(TAG, e, "ERROR. initReader() - Failed to get report rssi mode");
		}
		
		loadContinuousMode();
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initReader()");
	}
	
	// Initialize Widgets
	@Override
	protected void initWidgets() {
		super.initWidgets();

		txtSpeed = (TextView) findViewById(R.id.speed);
		txtSpeed.setText(R.string.init_tps);
		layoutOption = (LinearLayout) findViewById(R.id.inventory_option);
		layoutOption.setBackgroundColor(getColor(getApplicationContext(), R.color.message_background));
		txtPowerGain = (TextView) findViewById(R.id.inventory_power_gain);
		txtPowerGain.setOnClickListener(this);
		txtDisplayPC = (TextView) findViewById(R.id.inventory_display_pc);
		txtDisplayPC.setOnClickListener(this);
		txtContinuousMode = (TextView) findViewById(R.id.inventory_continuous_mode);
		txtContinuousMode.setOnClickListener(this);
		txtReportRssi = (TextView) findViewById(R.id.inventory_report_rssi);
		txtReportRssi.setOnClickListener(this);
		txtOperationTime = (TextView) findViewById(R.id.inventory_operation_time);
		txtOperationTime.setOnClickListener(this);
		txtQuerySession = (TextView) findViewById(R.id.inventory_query_session);
		txtQuerySession.setOnClickListener(this);
		txtQueryTarget = (TextView) findViewById(R.id.inventory_query_target);
		txtQueryTarget.setOnClickListener(this);
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initWidgets()");
	}

	// Enable/Disable Action Widgets
	@Override
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}

	@Override
	protected void clearWidgets() {
		super.clearWidgets();

		mSpeedTagCount = 0;
		mStartCalcTime = 0;
//		mLastTime = 0;
//		mTagSpeed = 0.0;

		txtSpeed.setText(String.valueOf(mSpeedTagCount)+" tcp");

		ASRLog.i(TAG, ASRLog.INFO, "INFO. clearWidgets()");
	}

	@Override
	protected void startInventory() {
		if (wakeLock!=null)
			wakeLock.acquire();	// Preventing screen sleep during inventory.

		adpTags.setReportTID(false);	// disable TID Text
		mIsRunRestart = mRestartTime > 0;
		super.startInventory();
	}

	@Override
	protected void stopInventory() {
		if (wakeLock!=null && wakeLock.isHeld())
			wakeLock.release();	// After the inventory is finished, cancels blocking the screen sleep mode.
		
		mIsRunRestart = false;
		super.stopInventory();
	}

	private Runnable mRestartHandler = new Runnable() {

		@Override
		public void run() {
			if (getReader().inventory() != ResultCode.NoError) {
				ASRLog.e(TAG, "ERROR. $mRestartHandler.run() - Failed to start inventory()");
				enableActionWidgets(true);
				return;
			}
			ASRLog.i(TAG, ASRLog.INFO, "INFO. $mRestartHandler.run()");
		}
	};
	
	private void saveConinuousMode() {
		SharedPreferences prefs = getSharedPreferences(MainActivity.APP_NAME, MODE_PRIVATE);
		SharedPreferences.Editor editor = prefs.edit();

		editor.putBoolean(KEY_LAST_CONTINUOUS_MODE, mContinuousMode);
		editor.apply();
		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. saveConinuousMode() - last Continuous Mode : [%s]", mContinuousMode);
	}
	
	private void loadContinuousMode() {
		SharedPreferences prefs = getSharedPreferences(MainActivity.APP_NAME, MODE_PRIVATE);
		mContinuousMode = prefs.getBoolean(KEY_LAST_CONTINUOUS_MODE, true);
		ASRLog.d(TAG, ASRLog.DEBUG, "DEBUG. loadContinuousMode() - last Continuous Mode : [%s]", mContinuousMode);
		
		if(getReader() != null) {
			try {
				getReader().setContinuousMode(mContinuousMode);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. loadContinuousMode() - Failed to set continuous mode");
			}
		}
	}

	/************************************************************
	 *  Screen On/Off Event
	 ************************************************************/
	private ScreenOnReceiver screenOnReceiver =null;
	class ScreenOnReceiver extends BroadcastReceiver {
		public void onReceive(Context context, Intent intent) {
			String action = intent.getAction();
			if (action.equals(Intent.ACTION_SCREEN_ON)) {
				R2000Model.getInstance().isTriggerEnable = true;
			} else if (action.equals(Intent.ACTION_SCREEN_OFF)) {
				R2000Model.getInstance().isTriggerEnable = false;
			}
		}
	}

	protected void useScreenOnOffMonitoring()
	{
		if(screenOnReceiver == null)
		{
			screenOnReceiver = new ScreenOnReceiver();
			IntentFilter filter = new IntentFilter();
			filter.addAction("android.intent.action.SCREEN_ON");
			filter.addAction("android.intent.action.SCREEN_OFF");
			registerReceiver(screenOnReceiver, filter);
		}
	}
}
