package jp.co.asterisk.asreader.l251g.view;

import java.util.Locale;

import jp.co.asterisk.asreader.l251g.support.adapter.LbtChannelAdapter;
import jp.co.asterisk.asreader.l251g.support.adapter.SpinnerAdapter;
import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.util.ResUtil;
import jp.co.asterisk.asreader.l251g.support.view.base.RfidPowerGainActivity;

import jp.co.asterisk.asreader.l251g.MainActivity;
import jp.co.asterisk.asreader.l251g.R;

import com.asreader.lib.rfid.exception.AsReaderException;
import com.asreader.lib.rfid.params.PowerRange;
import com.asreader.lib.rfid.type.AlgorithmType;
import com.asreader.lib.rfid.type.BuzzerState;
import com.asreader.lib.rfid.type.CommandType;
import com.asreader.lib.rfid.type.GlobalBandType;
import com.asreader.lib.rfid.type.LbtItem;
import com.asreader.lib.rfid.type.ResultCode;
import com.asreader.lib.rfid.type.VibratorState;
import com.asreader.lib.rfid.type.MaskType;

import com.asreader.lib.AsReader;
import com.asreader.lib.diagnostics.ASRLog;
import android.annotation.SuppressLint;
import android.app.ActionBar;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public class OptionActivity extends RfidPowerGainActivity implements OnClickListener {

	private static final String TAG = OptionActivity.class.getSimpleName();

	private static final int EVENT_LOAD_OPTION = 1000;
	private static final String KEY_BATTERY_INTERVAL = "battery_interval";
	private static final String KEY_MASK_TYPE = "mask_type";
	private static final int DEFAULT_BATTERY_INTERVAL = 10;
	private static final int DEFAULT_MASK_TYPE = 1;

	private TextView txtBuzzer;
	private TextView txtVibrator;
	private TextView txtInventoryTime;
	private TextView txtIdleTime;
	private TextView txtAutoOffTime;
	private TextView txtSleepTime;
	private TextView txtAlgorithm;
	private TextView txtQValue;
	private TextView txtMaxQ;
	private TextView txtMinQ;
	private TextView txtLinkProfile;
	private TextView txtGlobalBand;
	private TextView txtLogLevel;
	private TextView txtThresHold;
	private TextView txtBatteryInterval;
	private TextView txtMaskType;
	private Button btnSave;
	private Button btnDefault;
	private Button btnLbtChannel;
	private boolean mIsEnabled;
	private BuzzerState mBuzzer;
	private VibratorState mVibrator;
	private int mInventoryTime;
	private int mIdleTime;
	private int mAutoOffTime;
	private int mSleepTime;
	private AlgorithmType mAlgorithm;
	private int mQValue;
	private int mMaxQ;
	private int mMinQ;
	private int mLinkProfile;
	private int mBatteryInterval;
	private int mThreshold;
	private MaskType mMaskType;
	private int mPowerGain;
	private int mOperationTime;
	private PowerRange mPowerRange;
	private boolean mReportRSSI;
	private boolean mContinuousMode;
	private GlobalBandType mGlobalBandType;
	private int mLogLevel;
	private boolean mIsSaveTable;
	private FreqSlotItem[] mFreqSlots;
	private LbtItem[] mFreqTable;
	private boolean mIsSaved;
	private boolean mIsDefault;
	private LinearLayout layoutLBT;

	private Handler mEvent;
	
	private String[] mLogLevelText = new String[]{"ERROR", "INFO", "DEBUG", "API", "LIB1", "LIB2", "WATCH", "UTIL", "IO", "DUMP"};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_option);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
			actionBar.setCustomView(R.layout.activity_title);
			actionBar.setDisplayHomeAsUpEnabled(true);
			actionBar.setDisplayShowHomeEnabled(true);
		}

		TextView titleText = (TextView)findViewById(R.id.title_text);
		titleText.setText(R.string.option_name);
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			titleText.setTextColor(getColor(getApplicationContext(), R.color.black));

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		mIsSaveTable = false;
		mIsSaved = false;
		mIsDefault = false;

		// Create Reader
		createReader();

		Intent intent = getIntent();
		mBatteryInterval = intent.getIntExtra(KEY_BATTERY_INTERVAL, DEFAULT_BATTERY_INTERVAL);
		int code = intent.getIntExtra(KEY_MASK_TYPE, DEFAULT_MASK_TYPE);
		mMaskType = MaskType.valueOf(code);

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initReader();

		// Enable action widget
		enableActionWidgets(false);

		mEvent = new Handler();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		destroyReader();

		super.onDestroy();
	}

	@Override
	protected void onStart() {
		super.onStart();

		// Load Option
		WaitDialog.show(this, R.string.load_option);
		new Thread(mLoadOption).start();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onStart()");
	}

	@Override
	protected void onStop() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onStart()");
		super.onStop();
	}

	@Override
	protected void onResume() {
		super.onResume();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onResume()");
	}

	@Override
	protected void onPause() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onPause()");
		super.onPause();
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		switch (item.getItemId()) {
			case android.R.id.home:
				if(mIsSaved) {
					Intent intent = new Intent();
					intent.putExtra(KEY_BATTERY_INTERVAL, mBatteryInterval);
					intent.putExtra(KEY_MASK_TYPE, mMaskType.getCode());
					this.setResult(Activity.RESULT_OK, intent);
				} else {
					this.setResult(Activity.RESULT_CANCELED);
				}
				this.finish();

				return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	public void onBackPressed() {

		if(mIsSaved) {
			Intent intent = new Intent();
			intent.putExtra(KEY_BATTERY_INTERVAL, mBatteryInterval);
			intent.putExtra(KEY_MASK_TYPE, mMaskType.getCode());
			this.setResult(Activity.RESULT_OK, intent);
		} else {
			this.setResult(Activity.RESULT_CANCELED);
		}

		super.onBackPressed();
	}

	@Override
	public void onClick(View v) {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));
		mIsDefault = (v.getId() == R.id.option_default);

		switch (v.getId()) {
		case R.id.buzzer:
			showBuzzerDialog();
			break;
		case R.id.vibrator:
			showVibratorDialog();
			break;
		case R.id.inventory_time:
			showInventoryTimeDialog();
			break;
		case R.id.idle_time:
			showIdleTimeDialog();
			break;
		case R.id.auto_off_time:
			showAutoOffTimeDialog();
			break;
		case R.id.sleep_time:
			showSleepTimeDialog();
			break;
		case R.id.algorithm_type:
			showAlgorithmDialog();
			break;
		case R.id.q_value:
			showQValueDialog();
			break;
		case R.id.max_q:
			showMaxQDialog();
			break;
		case R.id.min_q:
			showMinQDialog();
			break;
		case R.id.link_profile:
			showLinkProfileDialog();
			break;
		case R.id.global_band:
			if (!getReader().isAvailableDefaultLinkProfile()){
				WaitDialog.show(this, R.string.load_frequency);
				new Thread(mLoadFrequencyTable).start();
			}else {
				Toast.makeText(getApplicationContext(), "This model does not support the [Global Band] function.",
						Toast.LENGTH_LONG).show();
			}
			break;
		case R.id.log_level:
			showLogLevelDialog();
			break;
		case R.id.thres_hold:
			if (!getReader().isAvailableDefaultLinkProfile()){
				Toast.makeText(getApplicationContext(), "This model does not support the [ThresHold] function.",
						Toast.LENGTH_LONG).show();
			}else {
				showThresHoldDialog();
			}
			break;
			case R.id.battery_interval:
			showBatteryIntervalDialog();
			break;
		case R.id.mask_type:
			showMaskTypeDialog();
			break;
		case R.id.btn_lbt_channel:
			WaitDialog.show(this, R.string.load_frequency);
			new Thread(mLoadFrequencyTable).start();
			break;
		case R.id.save:
			// Enable action widget
			enableActionWidgets(false);
			// Save Option
			WaitDialog.show(this, R.string.save_option);
			new Thread(mSaveOption).start();
			break;
		case R.id.option_default:
			ResultCode res;
			enableActionWidgets(false);

			WaitDialog.show(this, R.string.load_default_option);

			if ((res=getReader().defaultParameter()) != ResultCode.NoError) {
				// Save Option
				ASRLog.e(TAG, "ERROR. onClick(%s) - Failed to default parameter [%s]", ResUtil.getId(v.getId()), res);
				mIsDefault = false;
				enableActionWidgets(false);
				WaitDialog.hide();
				return;
			}
			break;
		}
	}

	@Override
	public void onCommandComplete(AsReader reader, CommandType command) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onCommandComplete (%s)", command);

		if (command == CommandType.DefaultParameter) {
			mBatteryInterval = DEFAULT_BATTERY_INTERVAL;
			mIsSaved = true;
			MainActivity.mUserConfigs.resetContents();
			MainActivity.mUserConfigs.Save();
			new Thread(mLoadOption).start();
		}
	}
	
	@Override
	protected void clearWidgets() {
	}
	
	// Asynchronously Load Frequency table
	private Runnable mLoadFrequencyTable = new Runnable() {

		@Override
		public void run() {
			ASRLog.i(TAG, ASRLog.INFO, "+++ INFO. $mLoadFrequencyTable.run()");

			Log.d(TAG, "mLoadFrequencyTable > start ");

			if(isJapaneseType()) {
				try {
					int[] slots = getReader().getLbtMask();
					mFreqSlots = new FreqSlotItem[slots.length];
					for (int i = 0; i < slots.length; i++) {
						String name = getReader().getLbtFreq(slots[i]);
						Log.d(TAG, "mLoadFrequencyTable > getLbtFreq [" + i + "] "+name + " / index : "+slots[i]);
						mFreqSlots[i] = new FreqSlotItem(slots[i], name);
					}


					mFreqTable = getReader().getLbt();

				} catch (AsReaderException e) {
					ASRLog.e(TAG, e, "ERROR. $mLoadOption.run() - Failed to get LBT table");
				}
			}
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
					showFrequencyDialog();
				}

			});
			mEvent.sendEmptyMessage(EVENT_LOAD_OPTION);
			ASRLog.i(TAG, ASRLog.INFO, "--- INFO. $mLoadOption.run()");
		}
		
	};

	// Asynchronous Load Option
	private Runnable mLoadOption = new Runnable() {

		@Override
		public void run() {
			ASRLog.i(TAG, ASRLog.INFO, "+++ INFO. $mLoadOption.run()");

			try {
				mBuzzer = getReader().getBuzzer();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mLoadOption.run() - Failed to get buzzer");
				mBuzzer = BuzzerState.Off;
			}
			try {
				mVibrator = getReader().getVibratorState();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mLoadOption.run() - Failed to get vibrator");
				mVibrator = VibratorState.Off;
			}
			try {
				mInventoryTime = getReader().getInventoryTime();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mLoadOption.run() - Failed to get inventory time");
				mInventoryTime = 150;
			}
			try {
				mIdleTime = getReader().getIdleTime();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mLoadOption.run() - Failed to get idle time");
				mIdleTime = 0;
			}
			try {
				mAutoOffTime = getReader().getAutoOffTime();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mLoadOption.run() - Failed to get auto off time");
				mAutoOffTime = 600;
			}

			try {
				mThreshold = getReader().getThreshold();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mLoadOption.run() - Failed to get auto off time");
				mThreshold = 50;
			}
			try {
				mSleepTime = getReader().getSleepTime();
			} catch (AsReaderException e2) {
				ASRLog.e(TAG, e2, "ERROR. $mLoadOption.run() - Failed to get sleep time");
				mSleepTime = 300;
			}
			
			try {
				mAlgorithm = getReader().getAlgorithm();
			} catch (AsReaderException e2) {
				ASRLog.e(TAG, e2, "ERROR. $mLoadOption.run() - Failed to get algorithm");
				mAlgorithm = AlgorithmType.DynamicQ;
			}
			
			try {
				mQValue = getReader().getQValue();
			} catch (AsReaderException e2) {
				ASRLog.e(TAG, e2, "ERROR. $mLoadOption.run() - Failed to get Q value");
				mQValue = 4;
			}
			
			try {
				mMaxQ = getReader().getMaxQ();
			} catch (AsReaderException e2) {
				ASRLog.e(TAG, e2, "ERROR. $mLoadOption.run() - Failed to get MAx Q value");
				mMaxQ = 15;
			}
			
			try {
				mMinQ = getReader().getMinQ();
			} catch (AsReaderException e2) {
				ASRLog.e(TAG, e2, "ERROR. $mLoadOption.run() - Failed to get Min Q value");
				mMinQ = 0;
			}
			
			try {
				mLinkProfile = getReader().getLinkProfile();
			} catch (AsReaderException e2) {
				ASRLog.e(TAG, e2, "ERROR. $mLoadOption.run() - Failed to get link profile");
				mLinkProfile = 1;
			}

			try {
				mGlobalBandType = getReader().getGlobalBand();

			} catch (AsReaderException e2) {
				ASRLog.e(TAG, e2, "ERROR. $mLoadOption.run() - Failed to get global band type");
				mGlobalBandType = GlobalBandType.Unknown;
			}
			
			// Get Reader Continuous Mode
			try {
				mContinuousMode = getReader().getContinuousMode();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. initReader() - Failed to get continuous mode");
			}
			
			// Get Report RSSI
			try {
				mReportRSSI = getReader().getReportRSSI();
			} catch (AsReaderException e1) {
				ASRLog.e(TAG, e1, "ERROR. initReader() - Failed to getReportRSSI");
			}

			// Get Power Gain
			try {
				mPowerRange = getReader().getPowerGainRange();
				mPowerGain = getReader().getPowerGain();
			} catch (AsReaderException e) {
				mPowerGain = mPowerRange.getMax();
			}
			// Get Operation Time
			try {
				mOperationTime = getReader().getOperationTime();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. initReader() - Failed to get operation time");
				mOperationTime = 0;
			}
			
			try {
				mMaskType = getReader().getMaskType();
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. initReader() - Failed to set mask type");
				mMaskType = MaskType.NoMask;
			}

			mLogLevel = ASRLog.getLogLevel();
			
			mIsSaveTable = false;
			mEvent.post(new Runnable() {

				@Override
				public void run() {

					displayBuzzer();
					displayVibrator();
					displayInventoryTime();
					displayIdleTime();
					displayAutoOffTime();
					displaySleepTime();
					displayAlgorithm();
					displayQValue();
					displayMaxQ();
					displayMinQ();
					displayLinkProfile();
					displayGlobalBandType();
					displayBatteryInterval();
					displayMaskType();
					displayThreshold();
					displayLogLevel();
					WaitDialog.hide();
					if(mIsDefault) {
						AlertDialog.Builder builder = new AlertDialog.Builder(OptionActivity.this);
						builder.setTitle("Success");
						builder.setMessage("Set to default successfully")
								.setCancelable(false)
								.setPositiveButton("OK", null);

						AlertDialog dlg = builder.create();
						dlg.show();
					}
					enableActionWidgets(true);
				}

			});
			mEvent.sendEmptyMessage(EVENT_LOAD_OPTION);
			ASRLog.i(TAG, ASRLog.INFO, "--- INFO. $mLoadOption.run()");
		}

	};

	// Asynchronous Save Option
	private Runnable mSaveOption = new Runnable() {

		@Override
		public void run() {
			ASRLog.i(TAG, ASRLog.INFO, "+++ INFO. $mSaveOption.run()");

			// Set Buzzer
			try {
				getReader().setBuzzer(mBuzzer);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set buzzer (%s)", mBuzzer);
			}
			// Set Vibrator
			try {
				getReader().setVibratorState(mVibrator);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set vibrator (%s)", mVibrator);
			}

			// Set Idle Time
			try {
				getReader().setIdleTime(mIdleTime);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set idle time (%d)", mIdleTime);
			}
			// Set Inventory Time
			try {
				getReader().setInventoryTime(mInventoryTime);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set inventory time (%d)", mInventoryTime);
			}
			// set Threshold
			try {
				getReader().setThreshold(mThreshold);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set threshold (%d)", mThreshold);
			}
			// Set Auto Off Time
			try {
				getReader().setAutoOffTime(mAutoOffTime);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set auto off time (%d)", mAutoOffTime);
			}
			// Set Sleep Time
			try {
				getReader().setSleepTime(mSleepTime);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set sleep time (%d)", mSleepTime);
			}
			// Set Algorithm
			try {
				getReader().setAlgorithm(mAlgorithm);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set algorithm (%S)", mAlgorithm);
			}
			// Set Q Value
			try {
				getReader().setQValue(mQValue);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set Q value (%d)", mQValue);
			}
			// Set Max Q Value
			try {
				getReader().setMaxQ(mMaxQ);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set Max Q value (%d)", mMaxQ);
			}
			// Set Min Q Value
			try {
				getReader().setMinQ(mMinQ);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set Min Q value (%d)", mMinQ);
			}
			// Set Link Profile
			try {
				getReader().setLinkProfile(mLinkProfile);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set Link profile (%d)", mLinkProfile);
			}

			// Set Mask Type
			try {
				getReader().setMaskType(mMaskType);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "--- ERROR. $mSaveOption.run() - Failed to set mask type");
			}

			// Set LBT Table. japan only
			if (mIsSaveTable) {
				try {
					getReader().setLbt(mFreqTable);
				} catch (AsReaderException e) {
					ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set LBT table");
				}
			}
				
			// Set Continuous mode
			try {
				getReader().setContinuousMode(mContinuousMode);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to setContinuousMode ");
			}
			// Set report rssi
			try {
				getReader().setReportRSSI(mReportRSSI);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to setReportRSSI ");
			}
			
			// Set power gain
			try {
				getReader().setPowerGain(mPowerGain);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set output power (power gain) [%d]", mPowerGain);
			}
			
			// Set operation time
			try {
				getReader().setOperationTime(mOperationTime);
			} catch (AsReaderException e) {
				ASRLog.e(TAG, e, "ERROR. $mSaveOption.run() - Failed to set operation time [%d]", mOperationTime);
			}

			MainActivity.mUserConfigs.setPowerGain(mPowerGain);
			MainActivity.mUserConfigs.setOperationTime(mOperationTime);
			MainActivity.mUserConfigs.setThreshold(mThreshold);
			MainActivity.mUserConfigs.setInventoryTime(mInventoryTime);
			MainActivity.mUserConfigs.setIdleTime(mIdleTime);
			MainActivity.mUserConfigs.setAutoOffTime(mAutoOffTime);
			MainActivity.mUserConfigs.setSleepTime(mSleepTime);
			MainActivity.mUserConfigs.setQValue(mQValue);
			MainActivity.mUserConfigs.setQMin(mMinQ);
			MainActivity.mUserConfigs.setQMax(mMaxQ);
			MainActivity.mUserConfigs.setMaskType(mMaskType);
			MainActivity.mUserConfigs.setLbtChannel(mFreqTable);
			MainActivity.mUserConfigs.setReportRssi(mReportRSSI);
			MainActivity.mUserConfigs.setLinkProfile(mLinkProfile);

			MainActivity.mUserConfigs.Save();

            MainActivity.mUserConfigs.setMaskDataUpdate(getReader(), mMaskType);

			ASRLog.setLogLevel(mLogLevel);
			
			mEvent.post(new Runnable() {

				@Override
				public void run() {
					WaitDialog.hide();
//					Intent intent = new Intent();
//					intent.putExtra(KEY_BATTERY_INTERVAL, mBatteryInterval);
//					intent.putExtra(KEY_MASK_TYPE, mMaskType.getCode());
//					intent.putExtra(KEY_REPORT_RSSI, mReportRSSI);
//					intent.putExtra(KEY_ACTION, getKeyAction());
//					setResult(Activity.RESULT_OK, intent);
					//finish();
					mIsSaved = true;

					AlertDialog.Builder builder = new AlertDialog.Builder(OptionActivity.this);
					builder.setTitle("Success");
					builder.setMessage("Saved successfully")
							.setCancelable(false)
							.setPositiveButton("OK", null);

					AlertDialog dlg = builder.create();
					dlg.show();
					enableActionWidgets(true);
				}

			});
			ASRLog.i(TAG, ASRLog.INFO, "+++ INFO. $mSaveOption.run()");
		}

	};



	// Initialize Reader
	@Override
	protected void initReader() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initReader()");
	}

	// Initialize Widgets
	protected void initWidgets() {
		txtBuzzer = (TextView) findViewById(R.id.buzzer);
		txtBuzzer.setOnClickListener(this);
		txtVibrator = (TextView) findViewById(R.id.vibrator);
		txtVibrator.setOnClickListener(this);
		txtInventoryTime = (TextView) findViewById(R.id.inventory_time);
		txtInventoryTime.setOnClickListener(this);
		txtIdleTime = (TextView) findViewById(R.id.idle_time);
		txtIdleTime.setOnClickListener(this);
		txtAutoOffTime = (TextView) findViewById(R.id.auto_off_time);
		txtAutoOffTime.setOnClickListener(this);
		txtSleepTime = (TextView) findViewById(R.id.sleep_time);
		txtSleepTime.setOnClickListener(this);
		txtAlgorithm = (TextView) findViewById(R.id.algorithm_type);
		txtAlgorithm.setOnClickListener(this);
		txtQValue = (TextView) findViewById(R.id.q_value);
		txtQValue.setOnClickListener(this);
		txtMaxQ = (TextView) findViewById(R.id.max_q);
		txtMaxQ.setOnClickListener(this);
		txtMinQ = (TextView) findViewById(R.id.min_q);
		txtMinQ.setOnClickListener(this);
		txtLinkProfile = (TextView) findViewById(R.id.link_profile);
		txtLinkProfile.setOnClickListener(this);
		txtGlobalBand = (TextView) findViewById(R.id.global_band);
		txtGlobalBand.setOnClickListener(this);
		txtBatteryInterval = (TextView) findViewById(R.id.battery_interval);
		txtBatteryInterval.setOnClickListener(this);
		txtMaskType = (TextView) findViewById(R.id.mask_type);
		txtMaskType.setOnClickListener(this);
		txtLogLevel = (TextView) findViewById(R.id.log_level);
		txtLogLevel.setOnClickListener(this);
		txtThresHold = (TextView) findViewById(R.id.thres_hold);
		txtThresHold.setOnClickListener(this);

		btnSave = (Button) findViewById(R.id.save);
		btnSave.setOnClickListener(this);
		btnDefault = (Button) findViewById(R.id.option_default);
		btnDefault.setOnClickListener(this);
		btnLbtChannel = (Button) findViewById(R.id.btn_lbt_channel);
		btnLbtChannel.setOnClickListener(this);
		layoutLBT = (LinearLayout) findViewById(R.id.lay_lbt);

		ASRLog.i(TAG, ASRLog.INFO, "INFO. initWidgets()");
	}

	// Enable/Disable Action Widgets
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);
		mIsEnabled = enabled;
		btnSave.setEnabled(enabled);
		btnDefault.setEnabled(enabled);

		if (isJapaneseType())
			layoutLBT.setVisibility(View.VISIBLE);
		else
			layoutLBT.setVisibility(View.GONE);

		ASRLog.i(TAG, ASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}
	
	private void displayBuzzer() {
		txtBuzzer.setText(mBuzzer.toString());
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayBuzzer() - [%s]", mBuzzer);
	}
	
	private void displayVibrator() {
		txtVibrator.setText(mVibrator.toString());
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayVibrator() - [%s]", mVibrator);
	}

	private void displayInventoryTime() {
		txtInventoryTime.setText(String.format(Locale.US, "%d ms", mInventoryTime));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayInventoryTime() - [%d]", mInventoryTime);
	}

	private void displayIdleTime() {
		txtIdleTime.setText(String.format(Locale.US, "%d ms", mIdleTime));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayIdleTime() - [%d]", mIdleTime);
	}

	private void displayAutoOffTime() {
		txtAutoOffTime.setText(String.format(Locale.US, "%d sec", mAutoOffTime));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayAutoOffTime() - [%d]", mAutoOffTime);
	}
	
	private void displaySleepTime() {
		txtSleepTime.setText(String.format(Locale.US, "%d sec", mSleepTime));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displaySleepTime() - [%d]", mSleepTime);
	}

	private void displayThreshold() {
		txtThresHold.setText(String.format(Locale.US, "%d", mThreshold));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. disThreshold() - [%d]", mThreshold);
	}
	
	private void displayAlgorithm() {
		txtAlgorithm.setText(mAlgorithm.toString());
		txtMaxQ.setEnabled(!(mAlgorithm == AlgorithmType.FixedQ));
		txtMinQ.setEnabled(!(mAlgorithm == AlgorithmType.FixedQ));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayAlgorithm() - [%s]", mAlgorithm);
	}
	
	private void displayQValue() {
		txtQValue.setText(String.format(Locale.US, "%d", mQValue));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayQValue() - [%s]", mQValue);
	}
	
	private void displayMaxQ() {
		txtMaxQ.setText(String.format(Locale.US, "%d", mMaxQ));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayMaxQ() - [%s]", mMaxQ);
	}
	
	private void displayMinQ() {
		txtMinQ.setText(String.format(Locale.US, "%d", mMinQ));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayMinQ() - [%s]", mMinQ);
	}
	
	private void displayLinkProfile() {
		txtLinkProfile.setText(String.format(Locale.US, "%d", mLinkProfile));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayLinkProfile() - [%s]", mLinkProfile);
	}

	private boolean isJapaneseType() {
		return mGlobalBandType == GlobalBandType.Japan125mW || mGlobalBandType == GlobalBandType.Japan1W ||
				mGlobalBandType == GlobalBandType.Japan250mW;
	}

	private boolean isETSI() {
		return mGlobalBandType == GlobalBandType.Europe || mGlobalBandType == GlobalBandType.Europe_B1 ||
				mGlobalBandType == GlobalBandType.Europe_B2 || mGlobalBandType == GlobalBandType.Europe_B3 ||
				mGlobalBandType == GlobalBandType.India || mGlobalBandType == GlobalBandType.Morocco;
	}

	private boolean isFCC() {
		return !(isETSI() || isJapaneseType());
	}
	
	private void displayGlobalBandType() {
		txtGlobalBand.setText(mGlobalBandType.toString());
		txtGlobalBand.setEnabled(isJapaneseType());
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayGlobalBandType() - [%s]", mGlobalBandType);
	}

	private void displayBatteryInterval() {
		txtBatteryInterval.setText(String.format(Locale.US, "%d sec", mBatteryInterval));
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayBatteryInterval() - [%d]", mBatteryInterval);
	}

	private void displayMaskType() {
		txtMaskType.setText(mMaskType.toString());
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayMaskType() - [%s]", mMaskType);
	}
	
	private void displayLogLevel() {
		txtLogLevel.setText(mLogLevelText[mLogLevel]);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. displayLogLevel() - [%s]", mLogLevelText[mLogLevel]);
	}

	private void showBuzzerDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for (BuzzerState item : BuzzerState.values()) {
			adapter.addItem(item.getCode(), item.toString());
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.buzzer);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = value.getCheckedItemPosition();
				mBuzzer = BuzzerState.valueOf(adapter.getValue(position));
				displayBuzzer();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showBuzzerDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showBuzzerDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showBuzzerDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mBuzzer.getCode());
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showBuzzerDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showBuzzerDialog()");
	}
	
	private void showVibratorDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for (VibratorState item : VibratorState.values()) {
			adapter.addItem(item.getCode(), item.toString());
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.vibrator);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = value.getCheckedItemPosition();
				mVibrator = VibratorState.valueOf(adapter.getValue(position));
				displayVibrator();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showVibratorDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showVibratorDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showVibratorDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mVibrator.getCode());
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showVibratorDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showVibratorDialog()");
	}

	private void showInventoryTimeDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
		final EditText value = (EditText) root.findViewById(R.id.value);

		final InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.inventory_time);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int nTxOnTime = 0;
				try {
					nTxOnTime = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR. showInventoryTimeDialog().$PositiveButton.onClick() - parseInt [%s]", value.getText().toString());
				}

				mInventoryTime = nTxOnTime;
				displayInventoryTime();

				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);

				ASRLog.i(TAG, ASRLog.INFO, "INFO. showInventoryTimeDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showInventoryTimeDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showInventoryTimeDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mInventoryTime));
				value.selectAll();
				value.requestFocus();
				value.postDelayed(new Runnable() {
					@Override
					public void run() {
						if(imm != null)
							imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
					}
				},30);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showInventoryTimeDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showInventoryTimeDialog()");
	}

	private void showIdleTimeDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		final InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.idle_time);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				//int idleTime = 0;
				int txOffTime = 0;
				try {
					txOffTime = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR. showIdleTimeDialog().$PositiveButton.onClick() - parseInt(%s)", value.getText().toString());
				}

				mIdleTime = txOffTime;
				displayIdleTime();

				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showIdleTimeDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showIdleTimeDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showIdleTimeDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mIdleTime));
				value.selectAll();
				value.requestFocus();
				value.postDelayed(new Runnable() {
					@Override
					public void run() {
						if(imm != null)
							imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
					}
				},30);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showIdleTimeDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showIdleTimeDialog()");
	}
	
	private void showSleepTimeDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		final TextView unit = (TextView) root.findViewById(R.id.unit);
		unit.setText(R.string.unit_sec);

		final InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.sleep_time);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, null);
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showSleepTimeDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showSleepTimeDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				final AlertDialog dlg = (AlertDialog) dialog;
				value.setText(String.format(Locale.US, "%d", mSleepTime));
				value.selectAll();
				value.requestFocus();
				Button positiveButton = dlg.getButton(AlertDialog.BUTTON_POSITIVE);
				positiveButton.setOnClickListener(new View.OnClickListener() {

					@SuppressLint("ShowToast")
					@Override
					public void onClick(View v) {
						int time;
						try {
							time = Integer.parseInt(value.getText().toString());
						} catch (Exception e) {
							time = 0;
						}
						mSleepTime = time;
						displaySleepTime();
						if(imm != null)
							imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
						dlg.dismiss();
					}
				});

				value.postDelayed(new Runnable() {
					@Override
					public void run() {
						if(imm != null)
							imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
					}
				},30);

				ASRLog.i(TAG, ASRLog.INFO, "INFO. showSleepTimeDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showSleepTimeDialog()");
	}

	private void showAutoOffTimeDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		final TextView unit = (TextView) root.findViewById(R.id.unit);
		unit.setText(R.string.unit_sec);
		final InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.auto_off_time);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, null);
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showAutoOffTimeDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showAutoOffTimeDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				final AlertDialog dlg = (AlertDialog) dialog;
				value.setText(String.format(Locale.US, "%d", mAutoOffTime));
				value.selectAll();
				value.requestFocus();

				Button positiveButton = dlg.getButton(AlertDialog.BUTTON_POSITIVE);
				positiveButton.setOnClickListener(new View.OnClickListener() {

					@SuppressLint("ShowToast")
					@Override
					public void onClick(View v) {
						int time;
						try {
							time = Integer.parseInt(value.getText().toString());
						} catch (Exception e) {
							time = 0;
						}
						mAutoOffTime = time;
						displayAutoOffTime();
						if(imm != null)
							imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
						dlg.dismiss();
					}
				});

				value.postDelayed(new Runnable() {
					@Override
					public void run() {
						if(imm != null)
							imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
					}
				},30);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showAutoOffTimeDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showAutoOffTimeDialog()");
	}

	private void showThresHoldDialog(){
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
		final EditText value = (EditText) root.findViewById(R.id.value);
		final TextView unit = (TextView) root.findViewById(R.id.unit);
		unit.setText(R.string.val);

		final InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.thres_hold);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mThreshold = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {
					mThreshold = 50;
				}

				if (mThreshold<50){
					Toast.makeText(getApplicationContext(), "'Thres Hold' should be greater than or equal 50.",
							Toast.LENGTH_LONG).show();
					return;
				}
				if (mThreshold>70){
					Toast.makeText(getApplicationContext(), "'Thres Hold' should be less than or equal 70.",
							Toast.LENGTH_LONG).show();
					return;
				}

				displayThreshold();

				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showThresHoldDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showThresHoldDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showThresHoldDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mThreshold));
				value.selectAll();
				value.requestFocus();
				value.postDelayed(new Runnable() {
					@Override
					public void run() {
						if(imm != null)
							imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
					}
				},30);

				ASRLog.i(TAG, ASRLog.INFO, "INFO. showThresHoldDialog()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showThresHoldDialog().onShow()");
	}

	private void showBatteryIntervalDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
		final TextView unit = (TextView) root.findViewById(R.id.unit);
		unit.setText(R.string.unit_sec);
		final EditText value = (EditText) root.findViewById(R.id.value);

		final InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.battery_interval);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					mBatteryInterval = Integer.parseInt(value.getText().toString());
				} catch (Exception e) {
					//ASRLog.e(TAG, "ERROR. showBatteryIntervalDialog().$PositiveButton.onClick() - parseInt(%s)", value.getText().toString());
					mBatteryInterval = 0;
				}

				displayBatteryInterval();
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showBatteryIntervalDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showBatteryIntervalDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				if(imm != null)
					imm.hideSoftInputFromWindow(value.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showBatteryIntervalDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				value.setText(String.format(Locale.US, "%d", mBatteryInterval));
				value.selectAll();
				value.requestFocus();
				value.postDelayed(new Runnable() {
					@Override
					public void run() {
						if(imm != null)
							imm.showSoftInput(value, InputMethodManager.SHOW_FORCED);
					}
				},30);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showBatteryIntervalDialog()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showBatteryIntervalDialog().onShow()");
	}

	private void showMaskTypeDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for (MaskType item : MaskType.values()) {
			adapter.addItem(item.getCode(), item.toString());
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.mask_type);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = -1;
				try {
					position = value.getCheckedItemPosition();
					mMaskType = MaskType.valueOf(adapter.getValue(position));
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR. showMaskTypeDialog().$PositiveButton.onClick() - MaskType.valueOf(%d)", position);
				}
				displayMaskType();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaskTypeDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaskTypeDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaskTypeDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mMaskType.getCode());
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaskTypeDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaskTypeDialog()");
	}

	private void showAlgorithmDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for (AlgorithmType item : AlgorithmType.values()) {
			adapter.addItem(item.getValue(), item.toString());
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.algorithm_type);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = -1;
				try {
					position = value.getCheckedItemPosition();
					mAlgorithm = AlgorithmType.valueOf(adapter.getValue(position));
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR. showAlgorithmDialog().$PositiveButton.onClick() - AlgorithmType.valueOf(%d)", position);
				}
				displayAlgorithm();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showAlgorithmDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showAlgorithmDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showAlgorithmDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mAlgorithm.getValue());
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showAlgorithmDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showAlgorithmDialog()");
	}

	private void showQValueDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);

		for(int n=0; n<=15; n++) {
			adapter.addItem(n, String.format(Locale.US, "%02d", n));
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.q_value);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = -1;
				try {
					position = value.getCheckedItemPosition();
					mQValue = adapter.getValue(position);
					// Start Q must be greater than or equal to Min Q and less than or equal to Max Q
					if(mAlgorithm == AlgorithmType.DynamicQ) {
						if(mQValue < mMinQ)
							mQValue = mMinQ;
						if(mQValue > mMaxQ)
							mQValue = mMaxQ;
					}
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR. showQValueDialog().$PositiveButton.onClick() - position(%d)", position);
				}
				displayQValue();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQValueDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQValueDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQValueDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mQValue);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showQValueDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showQValueDialog()");
	}

	private void showMaxQDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);

		for(int n=0; n<=15; n++) {
			adapter.addItem(n, String.format(Locale.US, "%02d", n));
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.max_q);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = -1;
				try {
					position = value.getCheckedItemPosition();
					mMaxQ = adapter.getValue(position);
					// Max Q must be greater than or equal to Start Q and Min Q
					if(mAlgorithm == AlgorithmType.DynamicQ) {
						if(mMaxQ < mQValue)
							mMaxQ = mQValue;
						if(mMaxQ < mMinQ)
							mMaxQ = mMinQ;
					}

				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR. showMaxQDialog().$PositiveButton.onClick() - position(%d)", position);
				}
				displayMaxQ();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaxQDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaxQDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaxQDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mMaxQ);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaxQDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMaxQDialog()");
	}

	private void showMinQDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);

		for(int n=0; n<=15; n++) {
			adapter.addItem(n, String.format(Locale.US, "%02d", n));
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.min_q);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = -1;
				try {
					position = value.getCheckedItemPosition();
					mMinQ = adapter.getValue(position);
					// Min Q must be less than or equal to Start Q and Max Q
					if(mAlgorithm == AlgorithmType.DynamicQ) {
						if(mMinQ > mQValue)
							mMinQ = mQValue;
						if(mMinQ > mMaxQ)
							mMinQ = mMaxQ;
					}

				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR. showMinQDialog().$PositiveButton.onClick() - position(%d)", position);
				}
				displayMinQ();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMinQDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMinQDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMinQDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mMinQ);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showMinQDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showMinQDialog()");
	}

	private void showLinkProfileDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);

		for(int n=0; n<=3; n++) {
			adapter.addItem(n, String.format(Locale.US, "%d", n));
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.link_profile);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = -1;
				try {
					position = value.getCheckedItemPosition();
					mLinkProfile = adapter.getValue(position);
				} catch (Exception e) {
					ASRLog.e(TAG, "ERROR. showLinkProfileDialog().$PositiveButton.onClick() - position(%d)", position);
				}
				displayLinkProfile();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLinkProfileDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLinkProfileDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLinkProfileDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mLinkProfile);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLinkProfileDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showLinkProfileDialog()");
	}

	private void showFrequencyDialog() {

		final LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView table = (ListView) root.findViewById(R.id.list);
		final LbtChannelAdapter adapter = new LbtChannelAdapter(this);
		for (FreqSlotItem item : mFreqSlots) {
			adapter.addItem(item.getSlot(), item.getName());
		}
		table.setAdapter(adapter);
		table.setChoiceMode(ListView.CHOICE_MODE_MULTIPLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.lbt_channel_title);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				mFreqTable = adapter.getTable();
				mIsSaveTable = true;
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLbtChannelDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				mIsSaveTable = false;
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLbtChannelDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				mIsSaveTable = false;
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLbtChannelDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				adapter.setTable(mFreqTable);
				adapter.notifyDataSetChanged();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLbtChannelDialog().onShow()");
			}
		});
		dialog.show();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. showLbtChannelDialog()");
	}
	
	private void showLogLevelDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		adapter.addItem(mLogLevelText);
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.log_level);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = value.getCheckedItemPosition();
				mLogLevel = adapter.getValue(position);
				displayLogLevel();
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLogLevelDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLogLevelDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLogLevelDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mLogLevel);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showLogLevelDialog().onShow()");
			}
		});
		dialog.show();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showLogLevelDialog()");
	}

	private class FreqSlotItem {
		private int mSlot;
		private String mFreqName;


		private FreqSlotItem(int slot, String name) {
			mSlot = slot;
			mFreqName = name;
		}

		private int getSlot() {
			return mSlot;
		}

		public String getName() {
			return mFreqName;
		}

		@Override
		public String toString() {
			return String.format(Locale.US, "%d, [%s]", mSlot, mFreqName);
		}
	}
}
