package jp.co.asterisk.asreader.l251g.view;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;

import jp.co.asterisk.asreader.l251g.R;
import com.asreader.lib.AsReader;
import com.asreader.lib.diagnostics.ASRLog;
import com.asreader.lib.rfid.exception.AsReaderException;
import com.asreader.lib.rfid.type.ActionState;
import com.asreader.lib.rfid.type.MemoryBank;
import com.asreader.lib.rfid.type.ResultCode;

import java.lang.ref.WeakReference;
import java.util.Locale;

import jp.co.asterisk.asreader.l251g.support.dialog.WaitDialog;
import jp.co.asterisk.asreader.l251g.support.util.ResUtil;
import jp.co.asterisk.asreader.l251g.support.view.base.MemoryActivity;

public class WriteMemoryActivity extends MemoryActivity {

	private static final String TAG = WriteMemoryActivity.class.getSimpleName();

	private TextView txtValue;

	private Button btnWrite;
	private String mValue;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_write_memory);
		ActionBar actionBar = getActionBar();
		if(actionBar != null) {
			actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
			actionBar.setCustomView(R.layout.activity_title);
			actionBar.setDisplayHomeAsUpEnabled(true);
			actionBar.setDisplayShowHomeEnabled(true);
		}
		TextView titleText = (TextView)findViewById(R.id.title_text);
		titleText.setText(R.string.write_memory_name);
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			titleText.setTextColor(getColor(getApplicationContext(), R.color.black));

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		createReader();

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initReader();

		InitTask task = new InitTask(this);
		task.execute();
	}

	@Override
	protected void onDestroy() {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		destroyReader();

		super.onDestroy();
	}

	@Override
	protected void onStart() {
		super.onStart();
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onStart()");
	}

	@Override
	protected void onStop() {
		
		if(getReader().getAction() == ActionState.WriteMemory)
			getReader().stop();

		// Rollback Mask
		exitMask();

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onStop()");
		super.onStop();
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onResume()");
	}

	@Override
	protected void onPause() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. onPause()");
		super.onPause();
	}
	
	@Override
	public void onClick(View v) {

		ASRLog.i(TAG, ASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));
		
		switch (v.getId()) {
		case R.id.write_value:
			showWriteValueDialog();
			break;
		case R.id.action_write:
			if (getReader().getAction() == ActionState.Stop) {
				writeMemory();
			} else {
				stopAction();
			}
			break;
		}
		super.onClick(v);
	}

	@Override
	public void onActionChanged(AsReader reader, ActionState action) {
		ASRLog.d(TAG, ASRLog.DEBUG, "EVENT. onActionChanged(%s)", action);

		enableActionWidgets(true);

		if (action == ActionState.Stop) {
			if (!isActionResult()) {
				outputMessage(R.string.write_memory_intro_message);
			}
		}
	}

	@Override
	public void onAccessResult(AsReader reader, ResultCode code, ActionState action, String epc, String data,
			float rssi, float phase, float frequency) {
		ASRLog.i(TAG, ASRLog.INFO, "EVENT. onAccessResult(%s, %s, [%s], [%s], %.2f, %.2f)", code, action, epc == null ? "" : epc,
				data == null ? "" : data, rssi, phase);

		setActionResult();

		if (code != ResultCode.NoError) {
			outputFailMessage(code.toString());
			return;
		}

		if(epc != null)
			outputSelection(epc.length() > 4 ? epc.substring(4) : epc);

		outputSuccessMessage(R.string.access_success);
		setRssi(rssi, phase, frequency);
	}

	@Override
	protected void clearWidgets() {
		super.clearWidgets();
		
		resetActionResult();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. clearWidgets()");
	}

	// Write Memory
	private void writeMemory() {
		clearWidgets();
		enableActionWidgets(false);
		outputSelection("");
		outputMessage(R.string.read_memory_read_message);

		resetActionResult();

		MemoryBank bank = getMemBank();
		int offset = getMemOffset();
		String data = getWriteValue();
		int time = getOperationTime();

		String password = getAccessPassword();
		try {
			getReader().setAccessPassword(password);
		} catch (AsReaderException e) {
			ASRLog.e(TAG, "ERROR. writeMemory() - Failed to set password [%s]", password);
		}

		try {
			getReader().setOperationTime(time);
		} catch (AsReaderException e) {
			ASRLog.e(TAG, "ERROR. writeMemory() - Failed to set operation time(%d)", time);
		}

		ResultCode res;
		if ((res = getReader().writeMemory(bank, offset, data)) != ResultCode.NoError) {
			ASRLog.e(TAG, "ERROR. writeMemory() - Failed to start write memory(%s, %d, [%s])", bank, offset, data);
			String msg = String.format(Locale.US, "%s : 0x%04X", res.toString(), res.getCode());
			Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
			enableActionWidgets(true);
			outputFailMessage(R.string.write_memory_fail_write_message);
			return;
		}
		ASRLog.i(TAG, ASRLog.INFO, "INFO. writeMemory()");
	}

	// Initialize Reader
	@Override
	protected void initReader() {
		super.initReader();
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initReader()");
	}

	// Initialize Widgets
	@Override
	protected void initWidgets() {
		super.initWidgets();

		ScrollView scrlParams = (ScrollView) findViewById(R.id.write_params);
		scrlParams.setBackgroundColor(getColor(getApplicationContext(), R.color.message_background));
		
		// Value EditText
		txtValue = (TextView) findViewById(R.id.write_value);
		txtValue.setOnClickListener(this);
		setWriteValue("");
		// Write Button
		btnWrite = (Button) findViewById(R.id.action_write);
		btnWrite.setOnClickListener(this);
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. initWidgets()");
	}

	// Enable/Disable Action Widgets
	@Override
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);

		btnWrite.setEnabled(isEnabledWidget(enabled, ActionState.WriteMemory));
		if (enabled) {
			switch (getReader().getAction()) {
			case WriteMemory:
				btnWrite.setText(R.string.action_stop);
				break;
			case Stop:
				btnWrite.setText(R.string.action_write_memory);
				break;
			default:
				break;
			}
		}
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}
	
	protected void setWriteValue(String value) {
		mValue = value;
		txtValue.setText(mValue);
		ASRLog.i(TAG, ASRLog.INFO, "INFO. setWriteValue([%s])", value);
	}
	
	protected String getWriteValue() {
		ASRLog.i(TAG, ASRLog.INFO, "INFO. getWriteValue() - [%s]", mValue);
		return mValue;
	}
	
	private void showWriteValueDialog() {
		if (!mIsEnabled)
			return;
		
		LinearLayout root = (LinearLayout)LinearLayout.inflate(this, R.layout.dialog_input_hex, null);
		final EditText writeValue = (EditText)root.findViewById(R.id.value);
		
		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.write_data);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				setWriteValue(writeValue.getText().toString());
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(writeValue.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showWriteValueDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {
			
			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(writeValue.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showWriteValueDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {
			
			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(writeValue.getWindowToken(), 0);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showWriteValueDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {
			
			@Override
			public void onShow(DialogInterface dialog) {
				writeValue.setText(mValue);
				writeValue.selectAll();
				writeValue.requestFocus();
				InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.showSoftInput(writeValue, InputMethodManager.SHOW_FORCED);
				ASRLog.i(TAG, ASRLog.INFO, "INFO. showWriteValueDialog().onShow()");
			}
		});
		dialog.show();
		
		ASRLog.i(TAG, ASRLog.INFO, "INFO. showWriteValueDialog()");
	}
	
	private static class InitTask extends AsyncTask<Void, Void, Boolean> {
		private final WeakReference<WriteMemoryActivity> mActivity;

		InitTask(WriteMemoryActivity activity) {
			mActivity = new WeakReference<>(activity);
		}

		@Override
		protected void onPreExecute() {
			WriteMemoryActivity activity = mActivity.get();
			WaitDialog.show(activity, R.string.init_mask);
			super.onPreExecute();
		}

		@Override
		protected Boolean doInBackground(Void... params) {
			// Initialize Mask
			WriteMemoryActivity activity = mActivity.get();
			activity.initMask();
			
			return true;
		}
		
		@Override
		protected void onPostExecute(Boolean result) {
			WriteMemoryActivity activity  = mActivity.get();
			activity.resetActionResult();

			activity.setMask(activity.getMask());

			// Enable action widget
			activity.enableActionWidgets(true);

			activity.outputMessage(R.string.write_memory_intro_message);
			WaitDialog.hide();
			super.onPostExecute(result);
		}
	}
}
